#!============================================================================
#! Matplotlib
#!============================================================================

import pylab, numpy

#!------------------------
#! Setting rcParams
#!------------------------
#! Changes to the basic pylab settings. This can be done in the pylab.rcParams dict.
print pylab.rcParams['figure.figsize']
print pylab.rcParams['text.usetex']
print pylab.rcParams['figure.dpi']
print pylab.rcParams['savefig.dpi']
print

#!================
#! Simple plotting
#!================
#! New figures
fig1 = pylab.figure()
fig2 = pylab.figure()

#! Activate a figure
pylab.figure(fig1.number)

#! Show and draw
# pylab.show()
# pylab.draw()

#! Clear a figure
pylab.clf()

#! Close figures
pylab.close()
pylab.close('all')

#! Interactive use
pylab.ion()
pylab.ioff()

#!------------------------
#! Plotting functions
#!------------------------
x = numpy.arange(0.0,10.,0.001)
y = numpy.sin(x)
pylab.plot(x,y, color='red', lw=2,label='data 1')
pylab.plot([4],[0],'ob',ms=20.,label='_nolegend_')
pylab.plot([6],[0],'ok',ms=20.,label='data 2')
pylab.xlim(1,9)
pylab.ylim(-1,1)
pylab.legend()
pylab.show()
print

#! Second y-axis
#!^^^^^^^^^^^^^^^
pylab.twinx()
pylab.plot(x+1,y*10., color='green',lw=3,label='data 3')
pylab.ylim(-15,15.)
pylab.show()
print

#! Exchange the data without creating a new figure
#!^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
pylab.figure()
h, = pylab.plot(x,y, color='red', lw=2,label='data 1')
pylab.xlim(1,9)
pylab.ylim(-1,1)
pylab.show()
h.set_data(y,x)
pylab.xlim(-1,1)
pylab.ylim(1,9)
pylab.show()
print

#!------------------------
#! Save figures to files
#!------------------------
# Many formats are suported: png, pdf, ps, svg...
pylab.savefig('filename.png')
print

#!---------------------------------------
#! Changing labels, ticks, titles etc...
#!---------------------------------------
#! Labels
#!^^^^^^^
pylab.figure()
pylab.plot(x,y, color='red', lw=2,label='data 1')
pylab.plot([4],[0],'ob',ms=20.,label='_nolegend_')
pylab.plot([6],[0],'ok',ms=20.,label='data 2')
pylab.xlim(1,9)
pylab.ylim(-1,1)
pylab.legend()
pylab.xlabel('X',fontsize=12)
pylab.ylabel('Y',color='blue',fontsize=25)
pylab.show()
print

#! Ticks, tickslabels
#!^^^^^^^^^^^^^^^^^^^
xticks = numpy.linspace(0,10,3)
xticks_labels = ['%g'%i for i in xticks]
pylab.xticks(xticks,xticks_labels,fontsize=14,color='green')
pylab.show()
print

#! Title
#!^^^^^^^
pylab.title('This is the title',fontsize=15,color='green',alpha=0.5)
pylab.show()
print

#!----------------------------
#! Multiple sub-plots
#!----------------------------
#! Using subplot
#!^^^^^^^^^^^^^^
# pylab.subplots_adjust: Tune the subplot layout
pylab.figure()
ax = pylab.subplot(2,1,1)
pylab.plot(x,y,'b--')
ax = pylab.subplot(2,1,2)
pylab.plot(y,x,'r--')
pylab.show()
print

#! Using axes
#!^^^^^^^^^^^
pylab.figure()

#! Whole figure axes
#!..................
ax_1 = pylab.axes([0,0,1,1])
pylab.plot(x,y,'g--')
pylab.show()
print

#! Smaller axes
#!.............
l = 0.1;b = 0.3; w = 0.4; h = 0.3
ax = pylab.axes([l,b,w,h])
pylab.plot(x,y,'k--')
pylab.title('This is the title of the smaller axes')
pylab.show()
print

#! Latex
#!.......
pylab.xlabel(r"$\sum_1^2$",fontsize=20)
pylab.ylabel(r'$\sigma$',fontsize=23,color='red')

pylab.show()
print

#! Text
#!.....
pylab.text(0.5, 0.7,'This is a text',
             horizontalalignment='center',
             verticalalignment='center',
             transform = ax_1.transAxes,
        )
pylab.show()
print

#!====================================
#! Shaded Regions
#!====================================
pylab.close('all')

# Make a blue box that is somewhat see-through
# and has a red border. 
pylab.fill([3,4,4,3], [2,2,4,4], 'b', alpha=0.2, edgecolor='r')
pylab.xlim(0,10)
pylab.ylim(0,10)
pylab.show()
print

#!------------------------
#! Fill below intersection
#!------------------------

def boltzman(x, xmid, tau):
    """
    evaluate the boltzman function with midpoint xmid and time constant tau
    over x
    """
    return 1. / (1. + numpy.exp(-(x-xmid)/tau))

pylab.clf()
x = numpy.arange(-6, 6, .01)
S = boltzman(x, 0, 1)
Z = 1-boltzman(x, 0.5, 1)
pylab.plot(x, S, x, Z, color='red', lw=2)
pylab.show()
print

pylab.clf()
def fill_below_intersection(x, S, Z):
    """
    fill the region below the intersection of S and Z
    """
    #find the intersection point
    ind = numpy.nonzero( numpy.absolute(S-Z)==min(numpy.absolute(S-Z)))[0]
    # compute a new curve which we will fill below
    Y = numpy.zeros(S.shape, dtype=numpy.float)
    Y[:ind] = S[:ind]  # Y is S up to the intersection
    Y[ind:] = Z[ind:]  # and Z beyond it
    pylab.fill(x, Y, facecolor='blue', alpha=0.5)

x = numpy.arange(-6, 6, .01)
S = boltzman(x, 0, 1)
Z = 1-boltzman(x, 0.5, 1)
pylab.plot(x, S, x, Z, color='red', lw=2)
fill_below_intersection(x, S, Z)
pylab.show()
print


#!========================
#! Colormaps
#!========================
pylab.close('all')
data = numpy.random.uniform(size=(20,20))


#! With evenly spaced axes
x = numpy.linspace(40,50,20)
y = numpy.linspace(10,50,20)
pylab.pcolor(x,y,data)
pylab.show()
print

#! Colorbar
pylab.colorbar()
pylab.show()
print

pylab.clf()

#! Variable axes
x = numpy.append(numpy.linspace(40,45,15),numpy.linspace(45,50,5))
pylab.pcolor(x,y,data)
pylab.show()
print

#! Flat shading
pylab.clf()
pylab.pcolor(x,y,data,shading='flat')
pylab.show()


#! Different colormaps in the same figure
pylab.clf()
pylab.subplot(2,1,1)
pylab.pcolor(x,y,data,shading='flat',cmap=pylab.cm.Greys)

pylab.subplot(2,1,2)
pylab.pcolor(x,y,data,shading='flat',cmap=pylab.cm.Greens)

pylab.show()
print

#!========================
#! Images
#!========================
img = pylab.imread('bccn.png')
pylab.figure()
pylab.imshow(img)
pylab.show()
print
pylab.axis('off')
pylab.show()
print

#!====================================
#! Arrows
#!====================================
pylab.close('all')


x = numpy.arange(10)
y = x

# Plot line
pylab.plot(x, y)

# Now lets make an arrow object
arr = pylab.Arrow(2, 2, 1, 1, edgecolor='white')

# Get the subplot that we are currently working on
ax = pylab.gca()

# Now add the arrow
ax.add_patch(arr)

# We should be able to make modifications to the arrow.
# Lets make it green.
arr.set_facecolor('g')

pylab.show()
print

#!======================
#! Elipses
#!======================

pylab.close('all')
from matplotlib.patches import Ellipse

NUM = 250

ells = [Ellipse(xy=pylab.rand(2)*10, width=pylab.rand(), height=pylab.rand(), angle=pylab.rand()*360)
        for i in xrange(NUM)]

fig = pylab.figure()
ax = fig.add_subplot(111, aspect='equal')
for e in ells:
    ax.add_artist(e)
    e.set_clip_box(ax.bbox)
    e.set_alpha(pylab.rand())
    e.set_facecolor(pylab.rand(3))

ax.set_xlim(0, 10)
ax.set_ylim(0, 10)

pylab.show()
print




#!======================
#! Interactive - Sliders
#!======================
from matplotlib.widgets import Slider, Button, RadioButtons

ax = pylab.subplot(111)
pylab.subplots_adjust(left=0.25, bottom=0.25)
t = numpy.arange(0.0, 1.0, 0.001)
a0 = 5
f0 = 3
s = a0*numpy.sin(2*numpy.pi*f0*t)
l, = pylab.plot(t,s, lw=2, color='red')
pylab.axis([0, 1, -10, 10])

axcolor = 'lightgoldenrodyellow'
axfreq = pylab.axes([0.25, 0.1, 0.65, 0.03], axisbg=axcolor)
axamp  = pylab.axes([0.25, 0.15, 0.65, 0.03], axisbg=axcolor)

sfreq = Slider(axfreq, 'Freq', 0.1, 30.0, valinit=f0)
samp = Slider(axamp, 'Amp', 0.1, 10.0, valinit=a0)

def update(val):
    amp = samp.val
    freq = sfreq.val
    l.set_ydata(amp*sin(2*pi*freq*t))
    draw()
sfreq.on_changed(update)
samp.on_changed(update)

resetax = pylab.axes([0.8, 0.025, 0.1, 0.04])
button = Button(resetax, 'Reset', color=axcolor, hovercolor='0.975')
def reset(event):
    sfreq.reset()
    samp.reset()
button.on_clicked(reset)

rax = pylab.axes([0.025, 0.5, 0.15, 0.15], axisbg=axcolor)
radio = RadioButtons(rax, ('red', 'blue', 'green'), active=0)
def colorfunc(label):
    l.set_color(label)
    pylab.draw()
radio.on_clicked(colorfunc)

pylab.show()
