import numpy, pylab
from NeuroTools.parameters import ParameterSpace
from NeuroTools.parameters import ParameterRange
from NeuroTools.sandbox import make_name
# import function that generates correlated spikes trains from a ParameterSet
# returns: cc - correlation coefficient, time_axis_cc - time axis for cross-correlation (for plotting), corrcoef - correlation coefficient between the two SpikeTrains
from examples_neurotools_parameters_tools import calc_cc

#! creating a ParameterSpace
p = ParameterSpace({})

#! adding fixed parameters
p.nu = 20. #! rate [Hz]
p.duration = 10000.

# adding ParameterRanges
p.c = ParameterRange([0.0,0.01,0.1,0.5])
p.jitter = ParameterRange([0.0,0.5,1.0,1.5])

# calculation of the ParameterSpace dimension and the labels of the parameters
# containing a range
dims, labels = p.parameter_space_dimension_labels()
print "dimensions: ", dims
print ' labels: ', labels

# creating a results array, with the dimensions of the ParameterSpace
corrcoef_results = numpy.empty(dims)
print corrcoef_results.shape

pylab.rcParams['figure.figsize'] = [16,14]
pylab.figure()

# scanning the ParameterSpace
for experiment in p.iter_inner():
    # calculation of the index in the space
    index = p.parameter_space_index(experiment)
    # perfomring the experiment
    cc,time_axis_cc, corrcoef = calc_cc(experiment)
    corrcoef_results[index] = corrcoef
    # plotting the cc's
    subplot_index = (dims[1]*index[0])+index[1]
    pylab.subplot(dims[0],dims[1],subplot_index+1)
    pylab.plot(time_axis_cc,cc)
    pylab.title(make_name(experiment,p.range_keys()))
    pylab.xlim(-30,30.)
    pylab.ylim(0,10.)

pylab.show()

# plot the results
pylab.matshow(corrcoef_results)
pylab.xticks(numpy.arange(0.5,dims[1]+0.5,1.0),[str(i) for i in p.jitter._values])
pylab.yticks(numpy.arange(0.5,dims[0]+0.5,1.0),[str(i) for i in p.c._values])
pylab.xlim(0,dims[1])
pylab.ylim(dims[0],0)
pylab.xlabel('jitter (ms)')
pylab.ylabel('correlation')
ax = pylab.colorbar()
ax.set_label('correlation')
pylab.draw()

pylab.show()
