
import os
os.environ['BRT_ADAPTER']='1'
#ticonfig --adapter=1 --od-getclocks

import stream
import kernels
import numpy

from time import time

def interlace(arrays):
    n = len(arrays)
    y,x = arrays[0].shape
    
    a = numpy.zeros((y*n,x),dtype=arrays[0].dtype)

    for i in xrange(n):
        a[i::n,:] = arrays[i]

    return a



k = kernels.__optimized4_matmult()

# dimensions for the streams

num_mrts = 4
num_floats = 4

# actual matrix dims for a, swapped for b
height,width = 4000,2000

a_width = width/num_floats
a_height = height/num_mrts 

b_width = height/num_floats
b_height = width/num_mrts

c_width = height/num_floats
c_height = height/num_mrts

# dims = numpy.array([width,height],dtype=numpy.uintc)

a_mrt_dims = numpy.array([a_width,a_height],dtype=numpy.uintc)
b_mrt_dims = numpy.array([b_width,b_height],dtype=numpy.uintc)
c_mrt_dims = numpy.array([c_width,c_height],dtype=numpy.uintc)

# number of elements 
a_N = a_width*num_floats*a_height
b_N = b_width*num_floats*b_height
c_N = c_width*num_floats*c_height

# allocate mrt streams

data_i1 = [numpy.random.normal(size=(a_N,)).astype(numpy.float32) for i in xrange(num_mrts)]
i1 = [stream.Stream_float4(a_mrt_dims) for i in xrange(num_mrts)]

#data_i2 = numpy.arange(0,N,dtype=numpy.float32)
data_i2 = [numpy.random.normal(size=(b_N,)).astype(numpy.float32) for i in xrange(num_mrts)]
i2 = [stream.Stream_float4(b_mrt_dims) for i in xrange(num_mrts)]

o1 = [stream.Stream_float4(c_mrt_dims) for i in xrange(num_mrts)]

#for i in numpy.arange(1000):
#    k.run(i1,i2,o1)

print "matmul width=%d,height=%d" % (width,height)

t1 = time()
for i in xrange(num_mrts):
    i1[i].read(data_i1[i])
    i2[i].read(data_i2[i])
t2 = time()

put_time = t2-t1

args = [a_width]+i1+i2+o1
# don't time first run, cuz it compiles.
k.run(*args)

t1 = time()
for i in xrange(6):
    k.run(*args)
t2 = time()

# last run is non-blocking, so don't count.
gpu_time = (t2-t1)/5.0

# this is execute plus pull_time
t1 = time()
z = [x.write() for x in o1]
t2 = time()

ans1 = interlace(z)

pull_time = t2-t1-gpu_time

print "->GPU transfer (s)", put_time
print "GPU compute time (s)", gpu_time
print "GPU-> transfer (s)", pull_time
print "GPU total (s)", put_time+gpu_time+pull_time

# CPU

arr1 = interlace([x.reshape(a_height,a_width*num_floats) for x in data_i1])
arr2 = interlace([x.reshape(b_height,b_width*num_floats) for x in data_i2])

t1 = time()
ans2 = numpy.dot(arr1,arr2)
print "CPU (s)", time()-t1

#t1 = time()
#out1d = numpy.dot(arr1d,arr2d)
#print "CPU double precision (s)", time()-t1


print "GPU==CPU:",numpy.allclose(ans1,ans2)





