import random
from basic_agents import BasicAgent, BasicAgentFactory
from game import Directions
from graph import Graph, Node, Edge, NoPathGraphException
from string import join


class GoAgent(BasicAgent):
    
    def choose_action(self, game_state):
        
        actions = game_state.getLegalActions(self.index)
        return random.choice(actions)
    
    def initialize(self, game_state):
        """This method handles the initial setup of the agent.
        """
        super(GoAgent, self).register_initial_state(game_state)
        self.maze_graph, self.maze_nodes = self.create_maze_graph(game_state)
        self.foods_graph, self.foods_nodes, self.food_paths = self.create_food_graph(game_state)
        try:
            curr_paths
        except NameError:
            global curr_paths 
            curr_paths={}
        
    
    def create_maze_graph(self, game_state):
        """Create a graph representation of the non-wall positions"""
        walls = game_state.getWalls()
        width, height = walls.width, walls.height
        
        nodes = {}
        for i in range(width):
            for j in range(height):
                if not walls[i][j]:
                    nodes[(i,j)] = Node((i,j))
        
        edges = []
        for (i,j), node in nodes.iteritems():
            if (i+1,j) in nodes:
                edges.append(Edge(nodes[(i,j)], nodes[(i+1,j)]))
                edges.append(Edge(nodes[(i+1,j)], nodes[(i,j)]))
            if (i,j+1) in nodes:
                edges.append(Edge(nodes[(i,j)], nodes[(i,j+1)]))
                edges.append(Edge(nodes[(i,j+1)], nodes[(i,j)]))
                
        return Graph(edges), nodes
    
    def create_food_graph(self, game_state):
        """Create a graph composed of only enemy foods which we are supposed to eat
        
        The nodes in the graph will have edges to all accessible other nodes"""
        
        foods = self.get_food(game_state)
        
        nodes = {}
        for i in range(foods.width):
            for j in range(foods.height):
                if foods[i][j]:
                    nodes[(i,j)] = Node((i,j))
                  
        edges = []
        food_pairs = [(food1, food2) for (i,food1) in enumerate(nodes) 
                                     for (j,food2) in enumerate(nodes)
                                     if i < j]
        food_paths = {}
        for food1, food2 in food_pairs:
            path1to2 = self.shortest_path(food1,food2)
            path2to1 = [food1] + path1to2[:-1]
            path2to1.reverse()
            edges.append(Edge(nodes[food1], nodes[food2]))
            edges.append(Edge(nodes[food2], nodes[food1]))
            food_paths[(food1, food2)] = path1to2
            food_paths[(food2, food1)] = path2to1
        
        return Graph(edges), nodes, food_paths
    
    def get_path(self, src, dest):
        if (src, dest) in self.food_paths:
            return self.food_paths[(src, dest)]
        else:
            assert src in self.maze_nodes, "Position %s is not valid" % str(src)
            assert dest in self.maze_nodes, "Position %s is not valid" % str(dest)
            return self.shortest_path(src, dest)
    
    def shortest_path(self, src, dest):
        """Returns the path with the shortest length from src to dest.
        
        src and dest are tuples indicating cell locations. path is a list of tuples.
        Ex: shortest_path((1,2), (2,4)) may return a path which is [(1,3), (2,3), (2,4)]
        It includes the destination but excludes the source
        """
        edges_on_path, path_weight = self.maze_graph.shortest_path(self.maze_nodes[src], self.maze_nodes[dest])
        path = [edge.head.position for edge in edges_on_path]
        return path
    
    def convert_next_position_to_direction(self, current_pos, next_pos):
        '''Converts a move given with 2 (neighbor) position to a Direction'''
        assert self.are_consecutive_positions(current_pos, next_pos) ,\
         "Positions are not consecutive"
        if current_pos[0] - next_pos[0] == 1:
            return Directions.WEST
        elif current_pos[0] - next_pos[0] == -1:
            return Directions.EAST
        elif current_pos[1] - next_pos[1] == 1:
            return Directions.SOUTH
        elif current_pos[1] - next_pos[1] == -1:
            return Directions.NORTH
    
    def are_consecutive_positions(self, pos1, pos2):
        '''
        Returns True if the 2 given positions are consecutive False when 
        They are not.
        '''
        if abs(pos1[0] - pos2[0]) + \
        abs(pos1[1] - pos2[1]) == 1    :
            return True
        else:
            return False
        
        
    def convert_list_of_positions_to_directions(self,list_positions):
        '''
        Takes a list of consecutive positions and converts it into a list
        of directions for the Agent
        '''
        for i in range(len(list_positions) - 1):
            assert self.are_consecutive_positions(list_positions[i],
                   list_positions[i + 1]), "Positions are not consecutive in"\
                   " the list at index %d" %i 
        list_directions = []
        for i in range(len(list_positions) - 1):
            list_directions.append(
                self.convert_next_position_to_direction(list_positions[i],
                                                        list_positions[i + 1]))
        return list_directions 
         
    def get_possible_paths(self, game_state, max_path_length):
        position=game_state.getAgentPosition(self.index)
        total_food=get_food_positions(game_state, self.is_red)
 
        global paths
        paths=[]
        remaining_foods=list(total_food)
        teammate_ind=self.teammate_index(game_state)
        if curr_paths.has_key(teammate_ind):
            for pos in curr_paths[teammate_ind]:
                if pos in remaining_foods:
                    remaining_foods.remove(pos)
                    #print self.index, teammate_ind, ' removing ', pos
        
        
        current_path=[]
        if remaining_foods:
            recursive_f(position, remaining_foods, current_path, [], self, max_path_length )
            
            
        return paths    
        def cmp_paths_function(x, y):
            """This function compares two objects in the global paths variable and order them by the food they contain
            """
            #print type(x[1]),type(y[1]), bool(x[1]>y[1])
            if x[1]>y[1]:
                return -1
            elif x[1]<y[1]:
                return 1
            else:
                return 0
        
        paths.sort(cmp_paths_function)
        #paths.sort(cmp=lambda x,y: x[1]>y[1])
        return paths


def get_food_positions(game_state, is_red):
    """Return a list of tuples, each one is a piece of our food
    """
    positions=[]
    # there is a function get_food(game_state) in BasicAgent, which returns our food
    if not is_red:
        food_map=game_state.getRedFood()
    else:  
        food_map=game_state.getBlueFood()

    for i in range(food_map.width):
        for j in range(food_map.height):
            if food_map.data[i][j]:
                positions.append((i,j))
    return positions


class NoPathsException(Exception):
      def __init__(self, value):
        self.parameter = value
      def __str__(self):
        return repr(self.parameter)

class GoAgent_eat(GoAgent):    

#    def register_initial_state(self.game_state):
 #       super(GoAgent_eat, self).register_initial_state
    
    def teammate_index(self, game_state):
        if self.is_red:
            ind=game_state.getRedTeamIndices()
        else:
            ind=game_state.getBlueTeamIndices()
        ind.remove(self.index)
        return ind[0]
        
        
        
    def calculate_path(self, game_state):
        all_our_food = get_food_positions(game_state, self.is_red)
        
        all_paths= self.get_possible_paths(game_state, 7)        
        
        index=1
        while not all_paths and index<30:
            all_paths=self.get_possible_paths(game_state, 7+7*index)
            index+=1
            
        try:
            curr_paths
        except NameError:
            global curr_paths 
            curr_paths={}

        if not all_paths:        
            raise NoPathsException, "we could not compute any paths, run awaaaaaay "
        curr_paths[self.index]=all_paths[0][0]
        self.num_of_curr_pos = 0
        #print curr_paths 
        return curr_paths
    
    def choose_action(self, game_state):
       try:
           curr_paths
       except NameError:
            global curr_paths 
            curr_paths={}

       if not curr_paths.has_key(self.index) or not curr_paths[self.index] or self.num_of_curr_pos ==len(curr_paths[self.index]):
           self.calculate_path(game_state)
           
       self.say(str(self.num_of_curr_pos))           
       next_pos = curr_paths[self.index][self.num_of_curr_pos]
        
       self.num_of_curr_pos+=1

           
       return self.convert_next_position_to_direction(game_state.getAgentPosition(self.index), next_pos)


#        if next_pos:
#            return self.convert_next_position_to_direction(game_state.getAgentPosition(self.index), next_pos)
#        else:
#            pass
class GoAgentFactory_eat(BasicAgentFactory):
    def get_agents_list(self):
        return [GoAgent_eat, GoAgent_eat]
    
class GoAgentFactory(BasicAgentFactory):
    def get_agents_list(self):
        return [GoAgent, GoAgent]



def recursive_f(initial_point, still_to_visit, current_path, eaten_food, agent_instance, max_path_length, recursion_level=0):

    can_get_somewhere_from_here=False            
    recursion_level+=1

    still_to_visit #.sort(cmp_function)
    not_changing_still_to_visit=list(still_to_visit)
    
    for a_food in not_changing_still_to_visit:
        food=tuple(a_food)
#        print "R"+str(recursion_level)+" eaten"+str(eaten_food)+" get_patING "+str(food)
        try:
            
            this_path=agent_instance.get_path(initial_point, food)
       # print len(current_path),
       # print len(this_path)
            if len(this_path)+len(current_path)   <= max_path_length:
                can_get_somewhere_from_here=True
                still_to_visit.remove(food)
                eaten_food_a=list(eaten_food)
                eaten_food_a.append(food)
                for pos in this_path:
                    current_path.append(pos)
                recursive_f(food, list(still_to_visit), list(current_path),list(eaten_food_a), agent_instance,max_path_length, recursion_level  )
                for _ in range(len(this_path)):
                    current_path.pop(-1)
        except NoPathGraphException, e:                
            print e
            pass
            #print current_path
                    
    if not can_get_somewhere_from_here and current_path and not (current_path in paths):
        """
        print "PATH("+str(len(eaten_food))+"):",
        for i in current_path:
            print i, " -> ",
        print
        """
        paths.append([current_path, len(eaten_food)])

 

