import random
from basic_agents import BasicAgent, BasicAgentFactory
from graph import Graph, Node, Edge
from string import join
class GoAgent(BasicAgent):
    
    def choose_action(self, game_state):
        
        actions = game_state.getLegalActions(self.index)
        return random.choice(actions)
    
    def register_initial_state(self, game_state):
        """This method handles the initial setup of the agent.
        """
        super(GoAgent, self).register_initial_state(game_state)
        self.maze_graph, self.maze_nodes = self.create_maze_graph(game_state)
    
    def create_maze_graph(self, game_state):
        """Create a graph representation of the non-wall positions"""
        walls = game_state.getWalls()
        width, height = walls.width, walls.height
        
        nodes = {}
        for i in range(width):
            for j in range(height):
                if not walls[i][j]:
                    nodes[(i,j)] = Node((i,j))
        
        edges = []
        for (i,j), node in nodes.iteritems():
            if (i+1,j) in nodes:
                edges.append(Edge(nodes[(i,j)], nodes[(i+1,j)]))
                edges.append(Edge(nodes[(i+1,j)], nodes[(i,j)]))
            if (i,j+1) in nodes:
                edges.append(Edge(nodes[(i,j)], nodes[(i,j+1)]))
                edges.append(Edge(nodes[(i,j+1)], nodes[(i,j)]))
                
        return Graph(edges), nodes
    
    def get_path(self, src, dest):
        return self.shortest_path(src, dest)
    
    def shortest_path(self, src, dest):
        """Returns the path with the shortest length from src to dest.
        
        src and dest are tuples indicating cell locations. path is a list of tuples.
        Ex: shortest_path((1,2), (2,4)) may return a path which is [(1,3), (2,3), (2,4)]
        It includes the destination but excludes the source
        """
        edges_on_path, path_weight = self.maze_graph.shortest_path(self.maze_nodes[src], self.maze_nodes[dest])
        path = [edge.head.position for edge in edges_on_path]
        return path
         
         
    def get_possible_paths(self, game_state):
        position=game_state.getAgentPosition(self.index)
        total_food=get_food_positions(game_state, self.is_red)
 
        global paths
        paths=[]
        remaining_foods=list(total_food)
        current_path=[]
        recursive_f(position, remaining_foods, current_path, [], self )
        
        def cmp_paths_function(x, y):
            """This function compares two objects in the global paths variable and order them by the food they contain ---> DOESNT WORKKKKK
            """
            print x[1],y[1], str(bool(x[1]>y[1]))
            if x[1]>y[1]:
                return 1
            elif x[1]<y[1]:
                return -1
            else:
                return 0
            
        paths.sort(cmp_paths_function)
        return paths


def get_food_positions(game_state, is_red):
    positions=[]
    if is_red:
        food_map=game_state.getRedFood()
    else:  
        food_map=game_state.getBlueFood()

    for i in range(food_map.width):
        for j in range(food_map.height):
            if food_map.data[i][j]:
                positions.append((i,j))
    return positions



class GoAgentFactory(BasicAgentFactory):

    def get_agents_list(self):
        return [GoAgent, GoAgent]

max_path_length=5



def recursive_f(initial_point, still_to_visit, current_path, eaten_food, agent_instance, recursion_level=0):
    can_get_somewhere_from_here=False            
    recursion_level+=1
#    def cmp_function(x, y):
#        """To sort the food by distance with the instace: food"""
#        dist_x=len(agent_instance.get_path(initial_point, x))
#        dist_y=len(agent_instance.get_path(initial_point, y))
#        if dist_x<dist_y:
#            return -1
#        elif dist_x>dist_y:
#            return 1
#        else:
#            return 0

 #   still_to_visit =     list(still_to_visit_i)
 #   current_path   = list(current_path_i)
 #   print current_path    
 #   eaten_food = list(eaten_food_i)
    still_to_visit #.sort(cmp_function)
    not_changing_still_to_visit=list(still_to_visit)
    
    for a_food in not_changing_still_to_visit:
        food=tuple(a_food)
       # print "R"+str(recursion_level)+" eaten"+str(eaten_food)+" CHECKING "+str(food),
        this_path=agent_instance.get_path(initial_point, food)
       # print len(current_path),
       # print len(this_path)
        if len(this_path)+len(current_path)   <= max_path_length:
            can_get_somewhere_from_here=True
            still_to_visit.remove(food)
            eaten_food_a=list(eaten_food)
            eaten_food_a.append(food)
            for pos in this_path:
                current_path.append(pos)
            recursive_f(food, list(still_to_visit), list(current_path),list(eaten_food_a), agent_instance, recursion_level  )
            for _ in range(len(this_path)):
                current_path.pop(-1)
            #print current_path
                    
    if not can_get_somewhere_from_here and current_path and not (current_path in paths):
        """
        print "PATH("+str(len(eaten_food))+"):",
        for i in current_path:
            print i, " -> ",
        print
        """
        paths.append([current_path, len(eaten_food)])

 

def which_food(path, is_red, game_state):
    foods=[]
    if is_red:
        food_map=game_state.getRedFood()
    else:  
        food_map=game_state.getBlueFood()
        
    total_food= get_food_positions(game_state, is_red)
   
    for pos in path:
        if pos in total_food:
            foods.append(pos)
    return foods
