import unittest
import basic_agents
from testingTools import get_mock_game
from go_agents import * #GoAgent, GoAgentFactory
from game import Directions
from graph import NoPathGraphException


class GoTestCase(unittest.TestCase):
    
    def setUp(self):
        self.create_test_data_1()
        
             
    def create_test_data_1(self):
        self.layout_txt = """%%%%%%%%%%%%%%%%%%%%
                             %       % % ....  2%
                             %  %%   %%%%%      %
                             %...% .   . %   .. %
                             %  %  . %   %%%%   %
                             %%%%% . %   % ..   %
                             %..     % . % . %%%%
                             % %   .            %
                             % ...%.         %  %
                             %   %%%%%%%     %. %
                             %   %%.   %     %..%
                             %   % .         %%%%
                             %     .         .  %
                             %1    %.. %%  % ...%
                             %%%%%%%%%%%%%%%%%%%%
                             """
        self.known_shortest_paths = {((1,1), (1,8)) : [(1,2), (1,3), (1,4),
                                                       (1,5), (1,6), (1,7), (1,8)], 
                                     ((5,9), (3,11)) : [(5,10), (5,11), (5,12), (5,13), (4,13),
                                                        (3,13), (2,13), (2,12), 
                                                        (2,11), (3,11)],
                                     ((16, 2), (17, 4)) : [(15, 2), (15, 3), (15, 4), (15, 5), (15, 6), (15, 7),
                                                           (16, 7), (17, 7), (17, 6), (17, 5), (17, 4)]}
        self.known_shortest_path_weights = {((1,1), (1,8)) : 7, 
                                            ((6,9), (3,11)) : 11,
                                            ((7,1), (4,6)) : 10}
    
    def test_red_blue_knowledge(self):
        """Test whether the agents know their team"""
        agents = [GoAgent(0), GoAgent(1)]
        lt, game = get_mock_game(self.layout_txt, agents)
        game._init_run()
        self.assertTrue(agents[0].is_red)
        self.assertFalse(agents[1].is_red)
        
    
    def test_shorthest_path(self):
        """Test the shortest path finding algorithms with some known data"""

        agents = [GoAgent(0), basic_agents.BasicAgent(1)]
        lt, game = get_mock_game(self.layout_txt, agents)
        game._init_run()
 
        
        for (src, dest), known_path in self.known_shortest_paths.iteritems():
            found_path = agents[0].shortest_path(src, dest)
            self.assertEqual(known_path, found_path)
            
        for (src, dest), known_path_weight in self.known_shortest_path_weights.iteritems():
            found_path = agents[0].shortest_path(src, dest)
            found_path_weight = len(found_path)
            self.assertEqual(known_path_weight, found_path_weight)
         
    def test_are_consecutive_positions(self):
        agent = GoAgent(0)
        assert agent.are_consecutive_positions((0,1),(0,0))
        assert agent.are_consecutive_positions((0,0),(1,0))
        assert not agent.are_consecutive_positions((0,0),(0,0))
        assert not agent.are_consecutive_positions((0,0),(0,2))
    
    def test_convert_next_position_to_direction(self):
        agent = GoAgent(0)
        assert agent.convert_next_position_to_direction((0,0),(0,1)) == Directions.NORTH
        assert agent.convert_next_position_to_direction((0,1),(0,0)) == Directions.SOUTH
        assert agent.convert_next_position_to_direction((1,0),(0,0)) == Directions.WEST
        assert agent.convert_next_position_to_direction((0,0),(1,0)) == Directions.EAST
 
    def test_get_path(self):
        """Test the get path finding algorithms with some known data"""

        agents = [GoAgent(0), basic_agents.BasicAgent(1)]
        lt, game = get_mock_game(self.layout_txt, agents)
        game._init_run()
        
        for (src, dest), known_path in self.known_shortest_paths.iteritems():
            found_path = agents[0].get_path(src, dest)
            self.assertEqual(known_path, found_path)
            
        for (src, dest), known_path_weight in self.known_shortest_path_weights.iteritems():
            found_path = agents[0].get_path(src, dest)
            found_path_weight = len(found_path)
            self.assertEqual(known_path_weight, found_path_weight)
    
    def test_no_path_exception(self):
        """Test whether the no path exception is raised correctly"""
        
        agents = [GoAgent(0), basic_agents.BasicAgent(1)]
        lt, game = get_mock_game(self.layout_txt, agents)
        game._init_run()
        src = (9, 13)
        dest = (11, 13)
        self.assertRaises(NoPathGraphException, agents[0].maze_graph.shortest_path, 
                                                agents[0].maze_nodes[src], 
                                                agents[0].maze_nodes[dest])
                   
        
    def test_get_food_positions(self):
        agents = [GoAgent(0), GoAgent(1)]
        lt, game = get_mock_game(self.layout_txt, agents)
        game._init_run()
        food_pos=get_food_positions(game.state, True)
        for p in food_pos:
            self.assertTrue(game.state.getRedFood().data[p[0]] [p[1]])
        
        
    def test_get_paths(self):
        agents = [GoAgent(0), GoAgent(1)]
        lt, game = get_mock_game(self.layout_txt, agents)
        game._init_run()
        print agents[0].get_possible_paths(game.state, max_path_length=15)[0]
        a=agents[1].get_possible_paths(game.state, max_path_length=15)
        
        self.assertTrue(True) # ???
        
    def test_convert_list_of_positions_to_directions(self):
        agent = GoAgent(0)
        assert agent.convert_list_of_positions_to_directions([(0,0),(0,1),(1,1),
        (1,2),(1,3),(1,2)]) == [Directions.NORTH, Directions.EAST, Directions.NORTH,
        Directions.NORTH, Directions.SOUTH]
        
if __name__=='__main__':
    unittest.main()
