import random
from basic_agents import BasicAgent, BasicAgentFactory
import numpy
import FeatureFunctions
import mazeLib
 
class ScoringAgent(BasicAgent):
    """Agent going for highest score concerning scoring function"""
        
    def __init__(self,index, scoringfunction,sightseeing_matrix):
        super(ScoringAgent,self).__init__(index)
        self.scoringfunction = scoringfunction
	self.sightseeing_matrix=sightseeing_matrix
              
    def choose_action(self, game_state):
        """Choose legal action with highest score..."""
        legalactions = numpy.array(game_state.getLegalActions(self.index))
        scores = self.scoringfunction(self.predict_pos(game_state,legalactions), game_state, self.index)
        #self.say(random.choice(['Smelling food at %i%i'%(target_coord[0],target_coord[1])]))	
        #self.say(str(game_state.getScore()))
        #index=(game_state.getRedTeamIndices()).index(self.index)
	my_team=game_state.getRedTeamIndices()
        if self.index not in my_team:
            my_team=game_state.getBlueTeamIndices()       
        index=(my_team).index(self.index)
	#print index
        # save gamestate for evaluation
	
        #f=open('/home/student/score.dat','w')
        #f.write(str(game_state.getScore()))
        #f.close()
        
        maxscore = scores.max()
        action_s = legalactions[scores == maxscore]
        #print legalactions,self.predict_pos(game_state,legalactions),scores,maxscore,action_s
	our_move=random.choice(action_s)
	self.sightseeing_matrix[index]+=0.1
	for x in range(self.sightseeing_matrix[index].shape[0]):
		for y in range(self.sightseeing_matrix[index].shape[1]):
			if self.sightseeing_matrix[index][x][y]>1.:
				self.sightseeing_matrix[index][x][y]=1.
	#print self.sightseeing_matrix
	current_position=game_state.getAgentPosition(self.index)
	self.sightseeing_matrix[index][current_position[0],current_position[1]]+=-0.2
        return our_move
	
    def predict_pos(self, game_state,legalactions):
    	"""Give resulting states of legal actions"""
        position=numpy.array(game_state.getAgentPosition(self.index))
    	transform={'Stop':[0,0],'North':[0,1],'East':[1,0],'South':[0,-1],'West':[-1,0]}
        states=[]
        for action in legalactions:
           states.append(position+transform[action])
        return states
 
class BasicScoringAgentFactory(BasicAgentFactory):
    """ Interface to construct ScoringAgentFactorys. For a new Factory inherit this one and overwrite the constructor"""	

    def __init__(self, **args):
        super(BasicScoringAgentFactory, self).__init__(**args)
	self.dc = mazeLib.DistanceCalculation()
    	self.scoringfunctions = [None, None]#[ScoringFunctions().constant_scoring_f, ScoringFunctions().nearest_manhattan_food_f]
    
    def get_agents_list(self):
        return [ScoringAgent, ScoringAgent]
	
    def getAgent(self, index):
        """Returns the agent for the provided index (customized with the corresponding scoringFunction)"""
        idx = self._agent_idx
        if idx >= len(self._agents_list):
            return ScoringAgent(index, self.scoringfunctions,self.sightseeing_matrix)        
        agent_class = self._agents_list[self._agent_idx]
        self._agent_idx += 1
        return agent_class(index,self.scoringfunctions[idx-1],self.sightseeing_matrix)
	
class ConstantScoringAgentFactory(BasicScoringAgentFactory):
    """ setting up 2 Agents where all states are assigned the same score: effectly this is a random angent"""

    def __init__(self, **args):
        super(ConstantScoringAgentFactory,self).__init__(**args)
	self.sightseeing_matrix=[None,None]
        self.scoringfunctions =[ScoringFunctions([0.3,0.3,0.3,0.3],self.dc,self.sightseeing_matrix).linear_feature_scoring_f,ScoringFunctions([0.3,0.3,0.3,0.3],self.dc,self.sightseeing_matrix).linear_feature_scoring_f]
		
class TwoDistsAgentFactory(BasicScoringAgentFactory):
    """ setting up 2 Agents, going for the state with closest path distance to food"""
	
    def __init__(self, **args):
        super(TwoDistsAgentFactory,self).__init__(**args)
        self.scoringfunctions =[ScoringFunctions(numpy.array([1.,0.,0.,0.]),self.dc,self.sightseeing_matrix).linear_feature_scoring_f,ScoringFunctions(numpy.array([1.,0.,0.,0.]),self.dc,self.sightseeing_matrix).linear_feature_scoring_f]
    
class ScoringFunctions(object):
    """ Contains scoring functions to rate (states)/(future legal actions)"""
    
    def __init__(self,weights,dc,sightseeing_matrix):
	self.dc=dc
	self.sightseeing_matrix=sightseeing_matrix
        self.features=FeatureFunctions.FeatureFunctions(self.dc,self.sightseeing_matrix)
        self.featurefuncvec=[self.features.single_nearest_food_feature,
	                                  self.features.single_nearest_friend_feature,
					  self.features.single_nearest_enemy_feature,
					  self.features.sightseeing_feature]
        self.weights=weights
	
	 
    def linear_feature_scoring_f(self,states,game_state,agentindex):
	#print self.weights
        featurevec=numpy.empty(len(self.weights))
        scores = numpy.empty(len(states))
        for stateindex,state in enumerate(states):
            for i, featurefunc in enumerate(self.featurefuncvec):
                featurevec[i]=featurefunc(state,game_state,agentindex)
	    #print featurevec
            scores[stateindex]=numpy.dot(self.weights,featurevec)
	#print scores
	
        return scores
		
    
#    def constant_scoring_f(self,states,game_state, agentindex):
#        """ Trivial scoring function returning 1"""
#        return numpy.ones(len(states))
#	
#    def nearest_manhattan_food_f(self, states, game_state, agentindex):  
#        """ Return as score the nearest manhattan distance to food"""
#        scores=[]
#        for state in states:
#            score=self.features.nearestFood(state, game_state,agentindex)
#            scores.append(score)
#        return numpy.array(scores) 
#	
#    def nearest_food_f(self,states,game_state,agentindex):
#        """Tets single_nearest_food_feature"""
#        scores=[]
#        for state in states:
#            score=self.features.single_nearest_food_feature(state,game_state,agentindex)
#            scores.append(score)
#        	#print scores
#        return numpy.array(scores) 
    
    
