import numpy
import mazeLib
import pickle

class FeatureFunctions(object):
    def __init__(self,dc,sightseeing_matrix):
         self.use_single_nearest_friend_feature=False
	 self.use_all_friends_feature=False
	 self.dc=dc
	 self.sightseeing_matrix=sightseeing_matrix
	 
    def manhattan_distance(self, pos1, pos2):
        return abs(pos1[0]-pos2[0])+ abs(pos1[1]-pos2[1])

    def nearestFood(self, position, game_state,agent_index):	
        #print position
	target_coord=position
	food=game_state.getBlueFood()
        current_nearest_dist=numpy.Inf 
	#print 'Food at:',
	for x in range(food.width):
	     for y in range(food.height):
	         if food[x][y]:
		     #print x,y,', ',
		     goal=(x,y)
		     dist=self.manhattan_distance(goal,position)	
		     if dist<current_nearest_dist:
		         current_nearest_dist=dist
			 target_coord=[x,y]
        #print 'Current nearest distance',current_nearest_dist
	
	return current_nearest_dist
	
	
    def single_nearest_food_feature(self,position,game_state,agent_index):
	"""
	    
	Calculates a float number 0<=feature<=1 reflecting the distance of the nearest food particle
	    
	"""
	    
	    
	#myfile=open('/home/student/summerschool/group1/game_state_2.pkl','wb')
	#pickle.dump(game_state,myfile)
	#myfile.close()
	
	#myfile=open('/home/student/summerschool/group1/game_state_2.pkl','rb')
	#new_game_state=pickle.load(myfile)
	#myfile.close()
	
	#assert(game_state==new_game_state)
	#print "assertion fine, break now"
	#raw_input()
	
	#dc = mazeLib.DistanceCalculation()
        #print game_state
	#print position, 
	nearest_distance,path=self.dc.get_nearest_food_distance(game_state,agent_index,(int(position[0]),int(position[1])))
	feature=1./float(nearest_distance+1.)
	#print feature
	maze_shape=numpy.array(game_state.getWalls().data).shape
	alternative_feature=float(nearest_distance)/self.manhattan_distance((0,0),maze_shape) 
	### is the origin of the maze 0,0 or 1,1?
	return feature

    def single_nearest_friend_feature(self,position, game_state,agent_index):
	"""
	    
	Calculates a float number 0<=feature<=1 reflecting the distance of the nearest friend 
	- very comparable to single_nearest_food_feature
	    
	"""
	
	if not self.use_single_nearest_friend_feature:
		if len(game_state.getLegalActions(agentIndex=agent_index))>3:
			self.use_single_nearest_friend_feature=True
		else:
			return 0.
	#dc = mazeLib.DistanceCalculation()
        #print game_state
	#nearest_distance,path=self.dc.get_nearest_friend_distance(game_state,agent_index,position)
	my_team=game_state.getRedTeamIndices()
	if agent_index not in my_team:
            my_team=game_state.getBlueTeamIndices()
	
	nearest_distance=None
	for index in my_team:
		if index!=agent_index:
			other_agent_position=game_state.getAgentPosition(index)
			#print position,other_agent_position
		        mh_distance=self.manhattan_distance(position,other_agent_position)
			#print mh_distance
			if nearest_distance is None or mh_distance<nearest_distance:
				nearest_distance=mh_distance
	if nearest_distance is None:
		return 1.

	feature=1.-1./float(nearest_distance+1)
		
	maze_shape=numpy.array(game_state.getWalls().data).shape
	alternative_feature=float(nearest_distance)/self.manhattan_distance((0,0),maze_shape) 
	### is the origin of the maze 0,0 or 1,1?
	return feature
	
    def single_nearest_enemy_feature(self,position,game_state,agent_index):
	"""
	    
	Calculates a float number 0<=feature<=1 reflecting the distance to the nearest 
	enemy reflecting the fact wether we are in our or their half - 
	behaviour should be homing on any enemy in our own half, and running away from any enemy in their half.
	    
	"""
	#dc = mazeLib.DistanceCalculation()
	#print (numpy.array(game_state.getWalls().data)).shape
	#print (numpy.array(game_state.getWalls().data)).shape[0]
	relative_x_coordination=(position[0])/(numpy.array(game_state.getWalls().data)).shape[0]
	if relative_x_coordination<0.5:
		we_are_in_our_half=True
	else:
		we_are_in_our_half=False
	#print
	#print position, we_are_in_our_half
	if not game_state.isOnRedTeam(agent_index):
		we_are_in_our_half=not we_are_in_our_half
        #print game_state
        if (0):
            nearest_distance,path=self.dc.get_nearest_enemy_distance(game_state,agent_index,[int(position[0]),int(position[1])])
	else:
            the_other_team=game_state.getRedTeamIndices()
            if agent_index in the_other_team:
                the_other_team=game_state.getBlueTeamIndices()
	
            nearest_distance=None
            #print the_other_team
            for index in the_other_team:
		other_agent_position=game_state.getAgentPosition(index)
		if other_agent_position is None:
			continue
		#print position,other_agent_position,
		mh_distance=self.manhattan_distance(position,other_agent_position)
		#print mh_distance
		if nearest_distance is None or mh_distance<nearest_distance:
			nearest_distance=mh_distance
            #print nearest_distance
	
	maze_shape=numpy.array(game_state.getWalls().data).shape
	if we_are_in_our_half:
	    if nearest_distance is None:
		return 0.	
	    feature=1./float(nearest_distance+1)
	    alternative_feature=float(nearest_distance)/self.manhattan_distance((0,0),maze_shape) 
	else:
	    if nearest_distance is None:
		return 1.	
	    feature=-1./float(nearest_distance+1)
	    alternative_feature=1.-float(nearest_distance)/self.manhattan_distance((0,0),maze_shape) 
	
	#print position, we_are_in_our_half,feature
	maze_shape=numpy.array(game_state.getWalls().data).shape
	alternative_feature=1.-float(nearest_distance)/self.manhattan_distance((0,0),maze_shape) 
	### is the origin of the maze 0,0 or 1,1?
	return feature
	
	
    def sightseeing_feature(self,position,game_state,agent_index):
        my_team=game_state.getRedTeamIndices()
        if agent_index not in my_team:
            my_team=game_state.getBlueTeamIndices()       
        index=(my_team).index(agent_index)
	#print index
        if self.sightseeing_matrix[index] is None:
		    self.sightseeing_matrix[index]=numpy.ones(numpy.array(game_state.getWalls().data).shape,dtype=numpy.float32)
	#print game_state.getRedTeamIndices()

	#print self.sightseeing_matrix[index]
	#print position
	#print int(position[0]),int(position[1])
	feature=self.sightseeing_matrix[index][int(position[0]),int(position[1])]
	#print feature
	#print
	return feature    
    

# List of features to be implemented:
# 1./distance_to_nearest_food (automatically normalized to be 0<x<1
# 1-1./distance_to_nearest_friend (repulsion to make agents go for different food-patches, make them less easier to be caught together etc.)
# 1-1./distance_to_nearest_enemy_while_in_enemies_half (stay away from the bad guy)
# 1./distance_to_nearest_enemy_while_in_our_half (chase/eat the bad guy)

# n_food/sum_of_distance_to_all_food (automatically normalized to be 0<x<1
# 1-(n_friendly_agents-1)/sum_of_distances_to_all_friends (repulsion to make agents go for different food-patches, make them less easier to be caught together etc.)
# 1-n_enemies/sum_of_distances_to_all_enemy_while_in_enemies_half (stay away from the bad guys)
# n_enemies/sum_of_distances_to_all_enemy_while_in_enemies_half (chase/eat the bad guy)

# alternative: Do all of the above using normalization to diagonal distance across the maze (so to say the maximum possible distance)
