import unittest
from game import Grid
from mazeLib import DistanceCalculation
import nikoGraph
import pickle
import numpy
import game
from capture import GameState

class ShortestPathTestCase(unittest.TestCase):
    def test_shortest_path1(self):
        # layout definieren
        # Grid
        # 0 1 1
        # 1 1 1
        g = Grid(3,2)
        g.data = [[0, 1],[0,0],[0,0]]
        #print  g
        dc = DistanceCalculation()
        path,dist=dc._shortest_path(g,[0,0],[1,1])
        #print path,dist
        self.failUnlessEqual(dist,2)
        self.failUnless(path==[(0,0),(1,0),(1,1)])
	path,dist = dc._shortest_path(g,[0,0],[2,1])
        #print path,dist
        self.failUnlessEqual(dist,3)
        self.failUnless(path in [[(0,0),(1,0),(1,1),(2,1)],[(0,0),(1,0),(2,0),(2,1)]])


    def test_shortest_path2(self):
        # layout definieren
        # Grid
        # 0 1 1
        # 1 1 1
        dc = DistanceCalculation()
        g = Grid(6,6)
        g.data = [[0,0,0,0,0,0],[0,1,1,1,1,0],[0,1,1,1,1,0],[0,0,1,1,1,0],[0,1,1,1,1,0],[0,0,0,0,0,0]]
        #print  g
        path,dist=dc._shortest_path(g,[4,5],[3,1])
	#print path,dist
        self.failUnlessEqual(dist,9)
        self.failUnless(path==[(4,5),(5,5),(5,4),(5,3),(5,2),(5,1),(5,0),(4,0),(3,0),(3,1)])


    def test_shortest_path3(self):
        # layout definieren
        # Grid
        # 0 1 1
        # 1 1 1
        dc = DistanceCalculation()
        g = Grid(2,2)
        g.data = [[1,0],[0,1]]
        #print  g
        path,dist = dc._shortest_path(g,[0,0],[1,1])
        self.failUnless(dist is numpy.Inf)
        self.failUnless(path is None)

    def test_position_out_of_bounds1(self):
        # layout definieren
        # Grid
        # 0 1 1
        # 1 1 1
        dc = DistanceCalculation()
        g = Grid(2,2)
        g.data = [[1,1],[1,1]]
        #print  g
        self.failUnlessRaises(AssertionError,dc._shortest_path,g,[2,2],[1,1])

    def test_position_out_of_bounds2(self):
        # layout definieren
        # Grid
        # 0 1 1
        # 1 1 1
        dc = DistanceCalculation()
        g = Grid(2,2)
        g.data = [[1,1],[1,1]]
        #print  g
        self.failUnlessRaises(AssertionError,dc._find_nearest_element,g,g,[5,5])


    def test_accepts_only_int(self):
        # layout definieren
        # Grid
        # 0 1 1
        # 1 1 1
        dc = DistanceCalculation()
        g = Grid(2,2)
        g.data = [[1,1],[1,1]]
        #print  g
        self.failUnlessRaises(AssertionError,dc._find_nearest_element,g,g,[1.,1.])



    def test_shortest_path_is_zero_works(self):
        # layout definieren
        # Grid
        # 0 1 1
        # 1 1 1
        g = Grid(3,2)
        g.data = [[0, 0],[0,0],[0,0]]
        #print  g
        dc = DistanceCalculation()
        path,dist=dc._shortest_path(g,[0,0],[0,0])
        #print path,dist
        self.failUnlessEqual(dist,0)
        self.failUnless(path==[(0,0),(0,0)])
	
        

class FindTargetTestCase(unittest.TestCase):
    def test_find_target1(self):
        g = Grid(3,2)
        g.data = [[0, 1],[0,0],[0,0]]
        
        dc = DistanceCalculation()
        targetgrid = Grid(3,2)
        targetgrid.data = [[0, 0],[0,1],[0,1]]
        path,dist=dc._find_nearest_element(g,targetgrid,[0,0])
        self.assertEqual(dist,2)
        correct_path = [(0,0),(1,0),(1,1)]
        self.assertEqual(path,correct_path)


class TestFoodCalculations(unittest.TestCase):	

    def test_nearest_food(self):
	myfile=open('game_state.pkl','rb')
	game_state=pickle.load(myfile)
	myfile.close()

        dc = DistanceCalculation()
        dist,ignore = dc.get_nearest_food_distance(game_state,0,list(game_state.getAgentPosition(0)))
        #dist = dc.get_nearest_food_distance(game_state,0,[0,0])
        self.assertEqual(dist,6)

# not yet 
class TestEnemyCalculations(unittest.TestCase):	
    def test_nearest_enemy(self):
        myfile=open('some_gamestate','rb')
        game_state=pickle.load(myfile)
        myfile.close()
        
        dc = DistanceCalculation()
        dist,ignore = dc.get_nearest_enemy_distance(game_state,0,list(game_state.getAgentPosition(0)))

    def test_nearest_enemy2(self):
        myfile=open('testcase1','rb')
        game_state=pickle.load(myfile)
        myfile.close()
        
        dc = DistanceCalculation()
        agent_pos = list(game_state.getAgentPosition(0))
        dist,ignore = dc.get_nearest_enemy_distance(game_state,0,(int(agent_pos[0]),int(agent_pos[1])))

        

class TestLookupTable(unittest.TestCase):	

    def test_lookup_table_init(self):
        dc = DistanceCalculation()
        g = Grid(2,2)
        g.data = [[0,0],[0,0]]
        dc._create_wall_lookup_table(g)
        self.assertEqual(dc._compute_maze_distance(g,[0,0],[1,1]),2)
        self.assertEqual(dc._compute_maze_distance(g,[0,0],[0,0]),0)
        self.assertEqual(dc._compute_maze_distance(g,[0,1],[1,1]),1)
        self.assertEqual(dc._compute_maze_distance(g,[1,0],[1,1]),1)
        self.assertEqual(dc._compute_maze_distance(g,[1,1],[1,1]),0)

    def test_lookup_table_consistent(self):
        g = Grid(3,1)
        g.data = [[0],[1],[0]]
        dc = DistanceCalculation()
        dc._create_wall_lookup_table(g)
        self.failUnlessEqual(dc.wall_lookup_table[0][0][0][0],0)
        self.failUnlessEqual(dc.wall_lookup_table[0][0][1][0],numpy.Inf)
        self.failUnlessEqual(dc.wall_lookup_table[0][0][2][0],numpy.Inf)
        self.failUnlessEqual(dc.wall_lookup_table[2][0][0][0],numpy.Inf)
        self.failUnlessEqual(dc.wall_lookup_table[2][0][1][0],numpy.Inf)
        self.failUnlessEqual(dc.wall_lookup_table[2][0][2][0],0)
        path,dist=dc._shortest_path(g,[0,0],[2,0])
        self.failUnlessEqual(dist,numpy.Inf)
        self.failUnless(path is None)
        dc._create_wall_lookup_table(g)

    def test_lookup_table_initspeed(self):
	#myfile=open('game_state.pkl','rb')
	#game_state=pickle.load(myfile)
	#myfile.close()
        w=5
        h=5
        g=numpy.random.uniform(0,1,[w,h])
        g[g<0.7]=0
        g[g!=0]=1
        gr=Grid(w,h)
        gr.data=g
        dc = DistanceCalculation()
        dc._create_wall_lookup_table(gr)
        #dist,ignore = dc.get_nearest_food_distance(game_state,0,list(game_state.getAgentPosition(0)))
        #dist = dc.get_nearest_food_distance(game_state,0,[0,0])
        #self.assertEqual(dist,6)

        
if __name__ == '__main__':
    unittest.main()
