class Node(object):
    """
    Node Class
    """
    pass

class Edge(object):
    """
    Edge Class
    """
    def __init__(self, nodes):
        self.node1, self.node2 = nodes

class WEdge(Edge):
    """
    Weighted Edge Class
    """
    def __init__(self, nodes, weights):
        super(WEdge, self).__init__(nodes)
        self.weights = weights

class Graph(object):
    """
    Graph Class
    """
    def __init__(self):
        self.nodes = {}
        self.edges = {}
        
    def add_node(self, name, node=None):
        if node == None:
            node = Node()
        self.nodes[name] = node
        
    def add_edge(self, name, nodes, weights=None):
        if weights == None:
            self.edges[name] = Edge(nodes)
        else:
            self.edges[name] = WEdge(nodes, weights)

    def neighbors(self, node_name):
        neigh = []
        for _, edge in self.edges.items():
            if edge.node1 == node_name:
                neigh.append(edge.node2)
            elif edge.node2 == node_name:
                neigh.append(edge.node1)
        return set(neigh)

    def directly_reachable(self, node_name):
        neigh = []
        for _, edge in self.edges.items():
            if edge.node1 == node_name:
                neigh.append(edge.node2)
        return set(neigh)
    
    def del_node(self, node_name):
        del self.nodes[node_name]
        for edge_name, edge in self.edges.items():
            if edge.node1 == node_name or edge.node2 == node_name:
                del self.edges[edge_name]
            
    def del_edge(self, edge_name):
        del self.edges[edge_name]
            
    def _get_nr_edges(self):
        return len(self.edges)
    
    def _get_nr_nodes(self):
        return len(self.nodes)
    
    nr_nodes = property(_get_nr_nodes) 
    nr_edges = property(_get_nr_edges) 
    
    def shortest_path2(self, source, target, cost):
        """Return distance from source to target, shortest path, 
        edges of the shortest path"""
        dist = {}
        previous = {}
        pending = set([])
        for node_name, _ in self.nodes.items():
            dist[node_name] = None
            previous[node_name] = None
            pending.add(node_name)
        dist[source] = 0
        finish = False
        
        while len(pending)>0 and finish == False:
            shortest_distance = None
            shortest_node = None
            for node_name in pending:
                if not dist[node_name] == None:
                    if shortest_distance == None or \
                    dist[node_name] < shortest_distance:
                        shortest_distance = dist[node_name]
                        shortest_node = node_name   
            if shortest_distance == None:
                finish = True
            elif shortest_node == target:
                pending.remove(shortest_node)
                finish = True
            else:
                pending.remove(shortest_node) 
                nodes_to_visit = \
                pending.intersection(self.directly_reachable(shortest_node))
                for node_name in nodes_to_visit:
                    dist_sh_node = None
                    for _, edge in self.edges.items():
                        if edge.node1 == shortest_node and \
                        edge.node2 == node_name:
                            if dist_sh_node == None or \
                            edge.weights[cost] < dist_sh_node:
                                dist_sh_node = edge.weights[cost]
                    alt = dist[shortest_node] + dist_sh_node
                    if alt < dist[node_name] or dist[node_name] == None:
                        dist[node_name] = alt
                        previous[node_name] = shortest_node    
        path = []
        path_edges = []
        node_name = target
        while previous[node_name] != None:
            path.insert(0, node_name)
            node_name = previous[node_name]
            edge_cost = None
            edge_name_short = None
            for edge_name, edge in self.edges.items():
                if edge.node1 == node_name and edge.node2 == path[0]:
                    if edge_cost == None or edge_cost > edge.weights[cost]:
                        edge_cost = edge.weights[cost]
                        edge_name_short = edge_name
            path_edges.insert(0, edge_name_short)
            
              
        return dist[target], path, path_edges
   
    def shortest_path(self, origin, destination, cost):
        node_costs = {}
        visited = set([origin])
        pending = []

        for node_name, _ in self.nodes.items():
            node_costs[node_name] = None

        node_costs[origin] = 0
        reach = self.directly_reachable(origin)
        for node in reach:
            for _, edge in self.edges.items():
                if edge.node1 == origin and edge.node2 == node:
                    if node_costs[node] == None or node_costs[node] \
                    > edge.weights[cost]:
                        node_costs[node] = edge.weights[cost]
        pending = pending + (list(reach))
        
        current = origin
        finished=False
        while not finished:
#            print "visited=", visited, " current=", current
            smallest_cost = None
            smallest_node = None
            for node in pending:
                if node not in visited:
                    if node_costs[node] < smallest_cost or \
                         smallest_cost == None:
                             smallest_cost = node_costs[node]
                             smallest_node = node
            if smallest_cost == None:
                finished = True
            else:
                current = smallest_node
                reach = self.directly_reachable(current)
                reach = reach - set(visited)
                for node in reach:
                    for _, edge in self.edges.items():
                        if edge.node1 == current and edge.node2 == node:
                            if node_costs[node] == None or \
                                node_costs[node] > node_costs[current] + \
                                edge.weights[cost]:
                                    node_costs[node] = node_costs[current] + \
                                    edge.weights[cost]
                pending = pending + list(reach)
                visited = visited.union(set([current]))
        print node_costs[destination]
                               
    
