"""
Date: 2009.09.05

Basic AgentFabric for Agents with hunger

ChangeLog
2009.09.02: Created agents to move only NORTH (Jose)
2009.09.03: Movement algorith (Anja)
2009.09.04: Python Style P9 and split Hungry behaviour into methods (Jose)

"""

# basic modules
# import random

# modules for controling Agents
# from basic_agents import BasicAgent, BasicAgentFactory
from basic_agents import BasicAgentFactory
# from drunk_Agents import Group2DrunkAgent

# this access directions
from game import Directions

#==============================================================================
# Alberto et al libraries
import networkx as nx
import paths
from Group2BasicAgent import Group2BasicAgent
#==============================================================================

class Group2AgentFactory_Hungry(BasicAgentFactory):
    """ Default constructor of agents """

    def get_agents_list(self):
        """ Create a list of the Agents described """

        return [HungryAgent, StopAgent]

class HungryAgent(Group2BasicAgent):
    """ Agent with hungry-only behaviour """
    
    def register_initial_state(self, game_state):
        """ This method handles the initial setup of the agent """
        super(HungryAgent, self).register_initial_state(game_state)

        # sets my init param, overwrite soon!
        self.my_pos = game_state.getAgentPosition(self.index)
        self.food_distance = None

    def get_food(self, game_state):
        """ gets a list of tuples with the food to eat """

        if self.is_red:
            return game_state.getBlueFood().asList()
            
        return game_state.getRedFood().asList()

    def choose_action(self, game_state):
        """ Agent movement: returns the values in actions, where
        actions are NORTH, SOUTH, WEST, EAST or STOP 
        and you can get them here:
        actions = game_state.getLegalActions(self.index)
        """
        # current position 
        self.my_pos = game_state.getAgentPosition(self.index)
        self.say('Hmmm!')

        # gets a list of tuples with all food locations
        food = self.get_food(game_state)

        # compute paths to food
        distances = []
        for i in food:
            distances.append(len(nx.shortest_path(self.maze_graph, 
                self.my_pos, i )))
         
        # min distance to food
        self.food_distance = min(distances)
        #print self.food_distance

        # take the index in food where dist is minimum
        destiny = food[distances.index(min(distances))]

        #self.coward_only = True
        #return Directions.STOP 
        return paths.action_to_reach( self.maze_graph, self.my_pos, destiny)

class StopAgent(Group2BasicAgent):
    """ Simply stops one of the agents """  
    
    def choose_action(self, game_state):
        """ Stop the Agent """
        return Directions.STOP

