"""
Agents file of group2
"""
import random
import paths
import networkx as nx
from basic_agents import BasicAgentFactory
from Group2BasicAgent import Group2BasicAgent

SCAPE_COUNTER_DECREASE = 1
SCAPE_COUNTER_INCREASE = 2
SCAPE_NOISE = 1.5
MAX_SCAPE_COUNTER = 2 * 30

MAX_TAG_COUNTER = 10

#SCAPE_COUNTER_DECREASE = 1
#SCAPE_COUNTER_INCREASE = 3
#MAX_SCAPE_COUNTER = 3 * 20

class Group2Agent_3(Group2BasicAgent):
    """
    Defensive agent
    """
    def choose_action(self, game_state):
        """
        Strategy of the agent
        """
        if self.my_mode == -1:
            self.say('Substitute Agent 5')
            return self.substitute.choose_action(game_state)
        else:
            self.my_pos = game_state.getAgentPosition(self.index)
            self.say('Agent 3')
            return paths.action_to_reach(self.maze_graph, \
                self.my_pos, self.target_pos)

    def components_score(self, game_state, comps):
        """
        Scores the subgraphs according to the food contents
        """
        score = 0
        if self.is_red:
            enemy_idxs = game_state.getBlueTeamIndices()
            our_food = game_state.getRedFood().asList()
        else:
            enemy_idxs = game_state.getRedTeamIndices()
            our_food = game_state.getBlueFood().asList()
        enemy_poss = [game_state.getInitialAgentPosition(index) for \
        index in enemy_idxs]
        for comp in comps:
            if not set(comp).intersection(enemy_poss):
                temp_score = len(set(our_food).intersection(comp))
                score += temp_score
        return score
    
    def register_initial_state(self, game_state):
        """This method handles the initial setup of the agent.
        You should override it if you have any business to do
        before the game begins (e.g., analyzing the maze).
        Make sure to invoke the super method when you override this.
        """
        super(Group2Agent_3, self).register_initial_state(game_state)
        n_comps = nx.number_connected_components(self.maze_graph)
        narrow_passages = []
        narrow_scores = []
        for node in self.maze_graph.nodes():
            if (node[0] <= self.my_maze.width/2 -1 and self.is_red) or \
            (node[0] > self.my_maze.width/2 -1 and not self.is_red):
                temp_graph = self.maze_graph.copy()
                temp_graph.remove_node(node)
                temp_comps = nx.connected_components(temp_graph)
                if len(temp_comps) > n_comps:
                    narrow_passages.append(node)
                    narrow_scores.append(self.components_score(game_state, \
                    temp_comps))
        self.target_pos = (-1, -1)
        self.my_mode = 0
        temp_score = 0
        for narrow_passage in narrow_passages:
            score = narrow_scores[narrow_passages.index(narrow_passage)]
            if score > temp_score:
                temp_score = score
                self.target_pos = narrow_passage
        if self.target_pos == (-1, -1): # switch to different strategy
            self.substitute = Group2Agent_5(self.index)
            self.substitute.register_initial_state(game_state)
            self.my_mode = -1

class Group2Agent_5(Group2BasicAgent):
    """
    Offensive agent, more careful regarding gohsts
    """
    def choose_action(self, game_state):
        """
        Strategy of the agent
        """
        self.my_pos = game_state.getAgentPosition(self.index)
        self.say('Agent 5')
        if self.my_pos == game_state.getInitialAgentPosition(self.index):
            self.scape_counter = 0
            self.tag_counter = 0
        if self.is_red:
            enemy_food = game_state.getBlueFood().asList()
        else:
            enemy_food = game_state.getRedFood().asList()
        dist = list()
        for food in enemy_food:
            dist.append(len(nx.shortest_path(self.maze_graph, \
                self.my_pos, food)))
            self.target_pos = enemy_food[dist.index(min(dist))]
        if self.tag_counter > 0:
            self.tag_counter -= 1
            self.target_pos = self.tagged_food
        current_action, current_path = \
            paths.action_and_path_to_reach(self.maze_graph, \
            self.my_pos, self.target_pos)
        enemy_positions = self.get_team_positions(game_state)
        if (enemy_positions[0] == None and enemy_positions[1] == None) or\
            (self.my_pos[0] <= self.my_maze.width/2 - 1 - 1 and self.is_red) \
            or \
            (self.my_pos[0] > self.my_maze.width/2 -1 + 1 and not self.is_red):
            self.scape_counter -= SCAPE_COUNTER_DECREASE
            if self.scape_counter < 0:
                self.scape_counter = 0
            return current_action
        else:
            path_blocked = False
            enemy_positions = set(enemy_positions).difference([None])
            if self.is_path_blocked(enemy_positions, current_path):
                path_blocked = True

            if path_blocked == False:
                self.scape_counter -= SCAPE_COUNTER_DECREASE
                if self.scape_counter < 0:
                    self.scape_counter = 0
                return current_action
            else:
                self.scape_counter += SCAPE_COUNTER_INCREASE * \
                    random.uniform(1, SCAPE_NOISE)
                if self.scape_counter > MAX_SCAPE_COUNTER:
                    if self.tag_counter == 0:
                        food = random.choice(enemy_food)
                        self.tagged_food = food
                        self.tag_counter = MAX_TAG_COUNTER
                        self.target_pos = food                
                    action, tag_path = \
                        paths.action_and_path_to_reach(self.maze_graph, \
                        self.my_pos, self.target_pos)
                    if len(tag_path) < self.tag_counter:
                        self.tag_counter = len(tag_path)
                    return action
                else:
                    best_food = None
                    temp_dist = 1000
                    for food in enemy_food:
                        best_action, best_path = \
                        paths.action_and_path_to_reach(self.maze_graph, \
                        self.my_pos, food)
                        if dist[enemy_food.index(food)] < temp_dist and\
                        not self.is_path_blocked(enemy_positions, best_path):
                            self.target_pos = food
                            temp_dist = dist[enemy_food.index(food)]
                            current_action = best_action
                    if best_food == None:
                        current_action = \
                        paths.action_to_reach(self.maze_graph, \
                        self.my_pos, \
                        game_state.getInitialAgentPosition(self.index))
                    return current_action
    
    def register_initial_state(self, game_state):
        """This method handles the initial setup of the agent.
        You should override it if you have any business to do
        before the game begins (e.g., analyzing the maze).
        Make sure to invoke the super method when you override this.
        """
        super(Group2Agent_5, self).register_initial_state(game_state)
        self.scape_counter = 0
        self.tag_counter = 0
        self.tagged_food = game_state.getInitialAgentPosition(self.index)

    def is_path_blocked(self, enemy_positions, path):  
        """
        Returns true if enemy_positions are in path (or close to it)
        """  
        path_blocked = False
        for enemy_position in enemy_positions:
            if enemy_position in path:
                path_blocked = True
            if (enemy_position[0], enemy_position[1]+1) in path:
                path_blocked = True
            if (enemy_position[0], enemy_position[1]-1) in path:
                path_blocked = True
            if (enemy_position[0]+1, enemy_position[1]) in path:
                path_blocked = True
            if (enemy_position[0]-1, enemy_position[1]) in path:
                path_blocked = True
        return path_blocked
            
class Group2AgentFactory_3(BasicAgentFactory):
    """
    Factory for the group2 agents
    """
    def get_agents_list(self):
        """Returns agents"""
#        return [AnjaAgent, AnjaAgent]
#        return [DefensiveReflexAgent, OffensiveReflexAgent]
        return [Group2Agent_3, Group2Agent_5]
