import networkx as nx
from game import Directions

def distance_heuristic(pos1, pos2):
    """Returns Manhattan Distance from a first to a second position"""
    return abs(pos2[0]-pos1[0]) + abs(pos2[1]-pos1[1])

def action_to_reach(graph, source, target):
    """Returns the action an agent has to take starting from source
    to move closer to target
    graph is the maze graph as in Group2Agent_3
    source, target are tuples (x,y)"""
    if source == target:
        return Directions.STOP
    path = nx.astar_path(graph, source, target, distance_heuristic)

#        path = nx.shortest_path(graph, source, target)
    if len(path) == 1:
        return Directions.STOP
    here = path[0]
    there = path[1]
    if there[0] > here[0]:
        return Directions.EAST
    elif there[1] > here[1]:
        return Directions.NORTH
    elif there[0] < here[0]:
        return Directions.WEST
    elif there[1] < here[1]:
        return Directions.SOUTH
    else:
        return Directions.STOP

def action_and_path_to_reach(graph, source, target):
    """Returns the action an agent has to take starting from source
    to move closer to target
    graph is the maze graph as in Group2Agent_3
    source, target are tuples (x,y)"""
    path = nx.astar_path(graph, source, target, distance_heuristic)
    if source == target:
        return Directions.STOP, path
#        path = nx.shortest_path(graph, source, target)
    if len(path) == 1:
        return Directions.STOP, path
    here = path[0]
    there = path[1]
    if there[0] > here[0]:
        return Directions.EAST, path
    elif there[1] > here[1]:
        return Directions.NORTH, path
    elif there[0] < here[0]:
        return Directions.WEST, path
    elif there[1] < here[1]:
        return Directions.SOUTH, path
    else:
        return Directions.STOP, path
    
def is_path_blocked(enemy_positions, path):  
    """
    Returns true if enemy_positions are in path (or close to it)
    """  
    path_blocked = False
    for enemy_position in enemy_positions:
        if enemy_position in path:
            path_blocked = True
        if (enemy_position[0], enemy_position[1]+1) in path:
            path_blocked = True
        if (enemy_position[0], enemy_position[1]-1) in path:
            path_blocked = True
        if (enemy_position[0]+1, enemy_position[1]) in path:
            path_blocked = True
        if (enemy_position[0]-1, enemy_position[1]) in path:
            path_blocked = True
    return path_blocked

