"""This is Group 3's meta agent."""
from basic_agents import BasicAgentFactory
from basic_agents import BasicAgent
from captureAgents import AgentFactory
import numpy
import pylab
#from game import Directions
from foraging_agents import ForagingAgent
from hunting_agents import HuntingAgent
from captureAgents import DefensiveReflexAgent
from captureAgents import OffensiveReflexAgent


class TwoFaceAgent(BasicAgent):
    """Group 3's meta agent"""
    
    def __init__(self, index, ghost_or_forager, infos):
        self.index = index
        self.is_red = None            
        self._display = None   # Access to the graphics (if available)
        self.infos = infos
        
        self.forager = ForagingAgent(index, self.infos)
        self.ghost = HuntingAgent(index, self.infos)
        assert(self.ghost.index == self.index)
        assert(self.forager.index == self.index)
        
        if ghost_or_forager == 'ghost':
            self.agent_type = 'ghost'
        if ghost_or_forager == 'forager':
            self.agent_type = 'forager'
        self.infos[self.index] = self.agent_type
            
    def choose_action(self, game_state):
        teamindices = self.get_team_indices(game_state, enemy_indices = False)
        teamindices.remove(self.index)
        our_other_agent_is = self.infos[teamindices[0]]
        
        # update tactics according to current score, or something else  
        current_score = self.get_score(game_state)
        if (self.infos['enemy_food_initial'] - \
           self.count_food(game_state, enemy_food = True)) / \
           float(self.infos['enemy_food_initial']) > \
           self.infos['defend_threshold']:
               if current_score >= self.infos['defend_score_th']:
                   self.infos['tactics'] = 'default'
               if current_score >= 3*self.infos['defend_score_th']:
                   self.infos['tactics'] = 'defend'
            
        elif self.count_food(game_state, enemy_food = True) / \
           float(self.infos['enemy_food_initial']) > \
           self.infos['forage_threshold'] and \
           self.count_food(game_state, enemy_food = False) / \
           float(self.infos['our_food_initial']) > \
           self.infos['defend_our_food_threshold']  and \
           current_score <= self.infos['forage_score_th']:
            self.infos['tactics'] = 'forage'
        else:
            self.infos['tactics'] = 'default'
            
        # check which tactics we use and assign agents accordingly        
        if self.infos['tactics'] == 'default':
            # if we have two ghosts, ghost that is not new will become forager 
            if our_other_agent_is == 'ghost' and self.agent_type == 'ghost' \
                   and game_state.getInitialAgentPosition(self.index) != \
                   game_state.getAgentPosition(self.index):
                self.say('must feeeed')
                self.agent_type = 'forager'
            
            # the respawned agent will always be a ghost
            if game_state.getAgentPosition(self.index) == \
                   game_state.getInitialAgentPosition(self.index):
                self.agent_type = 'ghost'
                self.say('feel my wrath!')
                game_state, 
            # if both agents are foragers, one should become ghost
            if our_other_agent_is == 'forager' and self.agent_type == 'forager':
                self.say('must feeeed')
                self.agent_type = 'forager'                
            
        elif self.infos['tactics'] == 'forage': 
            self.say('must feeeed')
            self.agent_type = 'forager' 
                
        elif self.infos['tactics'] == 'defend': 
            self.say('feel my wrath!')
            self.agent_type = 'ghost'                      
        
        self.infos[self.index] = self.agent_type

        if self.agent_type == 'ghost':
            return self.ghost.choose_action(game_state)
        elif self.agent_type == 'forager':
            return self.forager.choose_action(game_state)
        
    def count_food(self, game_state, enemy_food=True):
        n_food = 0
        for item in pylab.flatten(self.get_food(game_state, enemy_food=enemy_food)):
            if item:
                n_food += 1
        return n_food        
            
    def register_initial_state(self, game_state):
        """This method handles the initial setupself.infos = infos of the agent.
        You should override it if you have any business to do
        before the game begins (e.g., analyzing the maze).
        """
        self.is_red = game_state.isOnRedTeam(self.index)
        
        self.infos['enemy_food_initial'] = \
            self.count_food(game_state, enemy_food=True)
        
        self.infos['our_food_initial'] = \
            self.count_food(game_state, enemy_food=False)
        
        
        # Set up access to graphics if available
        import __main__
        if '_display' in dir(__main__):
            self._display = __main__._display
            self._txt_message = ''
            self._txt_graphics = None
            self._txt_pos = None
        
        self.forager.register_initial_state(game_state)
        self.ghost.registerInitialState(game_state)
        
        
    def _update_txt_message(self, game_state):
        """Moves the message on screen at the current position
        of the agent."""
        if self.agent_type == 'ghost':
            return self.ghost._update_txt_message(game_state)
        elif self.agent_type == 'forager':
            return self.forager._update_txt_message(game_state)
        
            
            

class TwoFaceAgentFactory(BasicAgentFactory):
    """The required Factory for this agent."""

    def __init__(self, **args):
        super(TwoFaceAgentFactory, self).__init__(**args)
        self._agents_list = self.get_agents_list()
        self.infos = {}
        self.infos['tactics'] = 'default'
        
        ## minimal enemy food percentage for dual forager mode 
        self.infos['forage_threshold'] = 0.3

        ## minimal own food percentage to allow two foragers
        self.infos['defend_our_food_threshold'] = 0.4

        ## if score < forage_score_th and minimal enemy food percentage 
        ## condition we go dual forage
        self.infos['forage_score_th'] = -50 #-3
        
        ## 1/2 eaten enemy food percentage that triggers dual defence mode 
        self.infos['defend_threshold'] = 0.9

        ## if score >= defend_score_th and eaten enemy food condition we go 
        ## dual defend
        self.infos['defend_score_th'] = 50 #2
        
        
    def get_agent_class(self):
        """Returns a list of Agent subclasses.
        If more agents are requested than provided, return a BasicAgent.
        This is the method to override to create new factories."""
        return 'ghost'
        #if self.last_agent_was == 'forager':
        #    self.last_agent_was = 'ghost'
        #    return 'ghost'
        #else:
        #    self.last_agent_was = 'forager'
        #    return 'forager'
        
    

    def getAgent(self, index):
        "Returns the agent for the provided index."
        agent_class = self.get_agent_class()
        return TwoFaceAgent(index, agent_class, self.infos)
        
        
        
        
        
class OffenseDefenseAgentFactory(AgentFactory):
    """Returns one offensive reflex agent and one defensive reflex agent"""
    
    
    def __init__(self, **args):
        AgentFactory.__init__(self, **args)
        self.offense = False
 
    def getAgent(self, index):
        self.offense = not self.offense
        if self.offense:
            return OffensiveReflexAgent(index)
        else:
            return DefensiveReflexAgent(index)

