""" path finding agent
    contains routines to get graph-based distance measures.
"""
import numpy
import dijkstra
# import pylab
import basic_agents

class Node(object):
    """Class construct nodes of a graph."""
    def __init__(self, xpos, ypos):
        self.position = (xpos, ypos)
        self.food = 0
        self.their_food = 0
        self.foodvalue = 0
    
    def get_food(self):
        """ get own food"""
        return self.food
        
    def get_their_food(self):
        """get enemy food"""
        return self.their_food
        
        
class GraphKnowingAgent(basic_agents.BasicAgent):  
    """Graph knowing agent is a basic agent with additional methods that 
    enable him to use knowledge of the game graph."""
    
    def __init__(self, index, infos = None):
        
        super(GraphKnowingAgent, self).__init__(index)
        self.index = index
        self.is_red = None            
        # Access to the graphics (if available)
        self._display = None
        if infos == None:
            infos = {}
        self.infos = infos
        self.nodes = None
        self.graph = None
        
    def construct_graph(self, game_state):
        """Construct the graph from the supplied game_state object. It will 
        be available in self.graph, node information is in self.nodes."""
    
        self.graph = {}
        self.nodes = {}
        wallinfo = game_state.getWalls()
        #sebi: killed the T below
        walls = numpy.array([k for k in wallinfo])
        
        for xpos in xrange(len(walls[:, 0])):
            for ypos in xrange(len(walls[0, :])):
                if not walls[xpos, ypos]:
                    self.nodes[(xpos, ypos)] = Node(xpos, ypos) 
                    pos_neighbors = [numpy.array([upos, vpos]) + 
                        numpy.array([xpos, ypos]) \
                    for (upos, vpos) in [(0, 1), (1, 0), (-1, 0), (0, -1)]]
                    neighbors = {}
                    for pos_n in pos_neighbors:      
                        try: 
                            if not walls[pos_n[0], pos_n[1]]:
                                neighbors[tuple(pos_n)] = 1
                        except IndexError: 
                            continue
                    self.graph[(xpos, ypos)] = neighbors
                        
    
    def get_shortest_path(self, pos_xy_from, pos_xy_to):
        """Compute the shortest path from coordinate pos_xy_from to pos_xy_to. 
        Output is a list of coordinate  tuples, it contains start and end 
        position."""
        return dijkstra.shortestPath(self.graph, pos_xy_from, pos_xy_to) 
    
    def get_neighbors(self, pos_xy, steps):
        """return a dictionary of neighbors that can be reached in a maximum 
        of steps steps, each with the associated number of steps.""" 
        
        assert(steps > 0)
        assert(type(pos_xy) == tuple)
        assert(len(pos_xy) == 2)
        
        neighbors = [[pos_xy]]
        neighbors.append(self.graph[pos_xy].keys())
        # neighbors     
        for _ in range(1, steps):
            newneighbors = []
            for xytup in neighbors[-1]:
                newneighbors.extend(self.graph[xytup].keys())
            for idx in xrange(len(neighbors)):
                for xytup in  neighbors[idx]:
                    while newneighbors.__contains__(xytup):
                        newneighbors.remove(xytup)
            neighbors.append( newneighbors )           

        neighb = {}
        for idx in xrange(len(neighbors)):
            for xytup in neighbors[idx]:
                neighb[xytup] = idx
                
        return neighb
    
    def get_fullgraph_bottleneck(self):
        """ Find bottleneck(s), i.e. nodes, where many shortest paths run 
        through in the whole graph"""
        tuple_list = list()
        keys = self.graph.keys()
        for key_1 in keys:
            for key_2 in keys:
                if key_1 == key_2:
                    continue
                else:
                    tuple_list.append(self.get_shortest_path(key_1, key_2))
        tuple_dict = self.graph
        for key in tuple_dict.keys():
            tuple_dict[key] = 0
            
        for idx in xrange(len(tuple_list)):
            for xytup in  tuple_list[idx]: 
                tuple_dict[xytup] += 1
        
        aidx = numpy.zeros(len(tuple_dict.items())) 
        tuples = []
        
        for idx in xrange(len(tuple_dict.items())):     
            aidx[idx] = tuple_dict.items()[idx][1] 
            tuples.append(tuple_dict.items()[idx][0])
                
        out_tuples = []        
        a_index = numpy.argsort(aidx)
        for idx in a_index[:-5:-1]:
            out_tuples.append(tuples[idx])
        return out_tuples                      
