import random
from basic_agents import BasicAgent, BasicAgentFactory
from subsystem import FoodSubsystem, EnemySubsystem
from keepgoing import KeepGoingSubsystem
import numpy
import random
import os
import traceback

DEBUG = os.getenv('TEAM4DEBUG')

class MasterAgentFactory(BasicAgentFactory):
    def get_agents_list(self):
        return [MasterAgent, MasterAgent]

 
class MasterAgent(BasicAgent):
    "Agent acting on behalf of team 4."
    def __init__(self, index):
        super(MasterAgent, self).__init__(index)
        self.systemlist = [FoodSubsystem(self), 
                            EnemySubsystem(self), 
                            KeepGoingSubsystem(self)
                            ] 
        self.weightlist = [1.0, -2.0,0.001]

    def register_initial_state(self, state):
        for system in self.systemlist:
            system.register_initial_state(state)
        
   # weight of enemy should be positive or zero if we are on our side of the board 
    
    def choose_action(self, game_state):
        actions = game_state.getLegalActions(self.index)

        # score is a list of the scores dictionary returned from different subsystems
        total_dir_score = dict((action,0) for action in actions)
        for i_subsys, subsys in enumerate(self.systemlist):
            # The subsystems get called here
            #score.append(subsys.assess(self,game_state,actions))

            try:
                ret = subsys.assess(game_state)
                print subsys.__class__.__name__, ret
                for action in total_dir_score:
                    total_dir_score[action] += ret.get(action, 0) * self.weightlist[i_subsys]
            except Exception, e:
                print 'an exception occured in assess'
                traceback.print_exc(e)
                if DEBUG:
                    raise
            
        max_val = max(total_dir_score.values())
        max_dirs = [key for key,val in total_dir_score.iteritems() if val==max_val]
        max_dir = random.choice(max_dirs)

        print 'our agent %03d goes %s' % (self.index, max_dir)
        return max_dir
    
                
        

