from numpy import inf

class DistCalc(object):
    """Calculate distance from src to dst.
    walls is a list of lists of booleans.
    src and dst are tuples like (x,y)
    """

    def __init__(self, walls, src, dst, limit=20):
        assert len(src) == 2
        assert len(dst) == 2
        
        self.walls = walls
        self.src = src
        self.dst = dst
        self.limit = limit
        self.distance = inf

        self.xsize = len(walls)
        self.ysize = len(walls[0])

        self.distances = dict(((x,y), inf)
                              for x in xrange(self.xsize)
                              for y in xrange(self.ysize))
        self.visited = set()
        self.distances[src] = 0
        self.visit(src)

    @staticmethod
    def walk_around(i,j):
        yield i-1, j
        yield i, j-1
        yield i+1, j
        yield i, j+1

    def isWall(self, i, j):
        try:
            return self.walls[i][j]
        except IndexError:
            # for whatever reason we sometimes go too far
            print 'isWall got %s,%s when size is %s,%s' % (i,j,self.xsize,self.ysize)
            return True

    def visit(self, pos):
        # import pdb; pdb.set_trace()
        dist = self.distances[pos]
        assert dist < inf

        #        if dist == 29:
        #            import pdb; pdb.set_trace()

        if pos == self.dst:
            self.distance = dist

        min_dist = inf
        min_nei = []
        
        for nei in self.walk_around(*pos):
            if self.isWall(*nei):
                continue
            newdist = dist + 1
            olddist = self.distances[nei]
            if newdist < olddist:
                #print 'changed distance[%s] %s -> %s' % (nei, olddist, newdist)
                self.distances[nei] = newdist
                min_nei.append(nei)

        self.visited.add(pos)
        #print 'visited: ', self.visited

        for nei in min_nei:
            self.visit(nei)

    def print_distances(self):
        for y in reversed(xrange(self.ysize)):
            for x in xrange(self.xsize):
                if self.isWall(x,y):
                    print ' [] ',
                else:
                    print '%4s' % self.distances[(x,y)],
            print
        print

def get_position(game, index):
    pos = game.getAgentPosition(index)
    if pos is None:
        return None
    i,j = pos
    return int(i), int(j)
