import unittest
import basic_agents
from testingTools import get_mock_game
from ra_agents import RAAgent
from numpy import inf

class DistanceTestCase(unittest.TestCase):
    def test_getEnemies(self):
        layout_txt = """%%%%%%%%%%%%%%%%
                        % %            %
                        %1%2    3      %
                        %%%%%%%%%%%%%%%%
                        """
        ag0 = RAAgent(0)
        # BasicAgent does nothing by default
        lt, game = get_mock_game(layout_txt, [ag0,
                                              basic_agents.BasicAgent(1),
                                              basic_agents.BasicAgent(2)
                                              ])
        game._init_run()
        self.assertEquals( ag0.getEnemies(game.state), [2] )


    def test_calc_distance_empty_field(self):
        layout_txt = """%%%%%%%%%%%%%%%%
                        %              %
                        %1 3          2%
                        %%%%%%%%%%%%%%%%
                        """
        ag0 = RAAgent(0)
        # BasicAgent does nothing by default
        lt, game = get_mock_game(layout_txt, [ag0,
                                              basic_agents.BasicAgent(1),
                                              basic_agents.BasicAgent(2),
                                              ])
        game._init_run()
        self.assertEquals( game.state.getAgentPosition(0), (1,1) )
        self.assertEquals( game.state.getAgentPosition(2), (3,1) )
        self.assertEquals( ag0.calWalkDist(game.state, 0), 0 )
        self.assertEquals( ag0.calWalkDist(game.state, 1), 13 )
        self.assertEquals( ag0.calWalkDist(game.state, 2), 2 )
        #self.assertEquals( ag0.scanEnemy(game.state), [(2,2)] )

    def test_calc_distance_empty_field2(self):
        layout_txt = """%%%%%%%%%%
                        %  2    3%
                        %1       %
                        %%%%%%%%%%
                        """
        ag0 = RAAgent(0)
        # BasicAgent does nothing by default
        lt, game = get_mock_game(layout_txt, [ag0,
                                              basic_agents.BasicAgent(1),
                                              basic_agents.BasicAgent(2)
                                              ])
        game._init_run()
        self.assertEquals( game.state.getAgentPosition(0), (1,1) )
        self.assertEquals( game.state.getAgentPosition(1), (3,2) )
        self.assertEquals( game.state.getAgentPosition(2), (8,2) )
        self.assertEquals( ag0.calWalkDist(game.state, 1), 3 )
        self.assertEquals( ag0.calWalkDist(game.state, 2), 8 )
        self.assertEquals( ag0.scanEnemy(game.state), [(2,8)] )

    def test_calc_distance_complicated1(self):
        layout_txt = """%%%%%%%%%%%%%%%%
                        %   %      %   %
                        % % % %  % %  %%
                        % % % %  % %   %
                        % % % %  % %  %%
                        % % % %  % % %%%
                        %1%3  %       2%
                        %%%%%%%%%%%%%%%%
                        """
        ag0 = RAAgent(0)
        # BasicAgent does nothing by default
        lt, game = get_mock_game(layout_txt, [ag0,
                                              basic_agents.BasicAgent(1),
                                              basic_agents.BasicAgent(2),
                                              ])
        game._init_run()
        self.assertEquals( game.state.getAgentPosition(0), (1,1) )
        self.assertEquals( game.state.getAgentPosition(2), (3,1) )
        self.assertEquals( ag0.calWalkDist(game.state, 0), 0 )
        self.assertEquals( ag0.calWalkDist(game.state, 1), 33 )
        self.assertEquals( ag0.calWalkDist(game.state, 2), 12)

    def test_calc_distance_complicated2(self):
        layout_txt = """%%%%%%%%%%%%%%%%
                        %              %
                        %              %
                        %              %
                        % %%%%%%       %
                        % %    %   %%%%%
                        %1%3          2%
                        %%%%%%%%%%%%%%%%
                        """
        ag0 = RAAgent(0)
        # BasicAgent does nothing by default
        lt, game = get_mock_game(layout_txt, [ag0,
                                              basic_agents.BasicAgent(1),
                                              basic_agents.BasicAgent(2),
                                              ])
        game._init_run()
        self.assertEquals( game.state.getAgentPosition(0), (1,1) )
        self.assertEquals( game.state.getAgentPosition(2), (3,1) )
        self.assertEquals( ag0.calWalkDist(game.state, 0), 0 )
        self.assertEquals( ag0.calWalkDist(game.state, 1), 19 )
        self.assertEquals( ag0.calWalkDist(game.state, 2), 18)


    def test_calc_distance_complicated3(self):
        layout_txt = """%%%%%%%%%%%%%%%%
                        %   %      %   %
                        % % % %  % % %%%
                        % %   %        %
                        % % % %  % %  %%
                        % % % % 1% % %%%
                        % %3  %       2%
                        %%%%%%%%%%%%%%%%
                        """
        ag0 = RAAgent(0)
        # BasicAgent does nothing by default
        lt, game = get_mock_game(layout_txt, [ag0,
                                              basic_agents.BasicAgent(1),
                                              basic_agents.BasicAgent(2),
                                              ])
        game._init_run()
        self.assertEquals( ag0.calWalkDist(game.state, 0), 0 )
        self.assertEquals( ag0.calWalkDist(game.state, 1), 7 )
        self.assertEquals( ag0.calWalkDist(game.state, 2),14 )        

if __name__=='__main__':
    unittest.main()
