import unittest
from bartosz_agents import GPAgent
from testingTools import get_mock_game
from gp import Node, ParamNode, ConstNode
from numpy import array, abs, sum

class EscapeTestCase(unittest.TestCase):

    def test_case(self):
        layout_txt = """%%%%%%%
                        %     %
                        %  1  %
                        %     %
                        %%%%%%%"""
        
        my_agent = GPAgent(0,None)
        addw = my_agent.addw
        delta = array([0,1])
        strategy = Node(addw, [ParamNode(0), ConstNode(delta)])
        
        my_agent.strategy_tree = strategy

        lt, game = get_mock_game(layout_txt, [my_agent])
        game._init_run()
        
        pos = array(game.state.getAgentPosition(0))
        game._one_game_step()
        # test that the agent really moved east!
        new_pos = array(game.state.getAgentPosition(0))

        self.assertAlmostEqual(sum(abs(pos-new_pos+delta),0), 0, 2)

        

    def test_case2(self):
        layout_txt = """%%%%%%%
                        %     %
                        %  1  %
                        %     %
                        %%%%%%%"""
        
        my_agent = GPAgent(0,None)
        addw = my_agent.addw
        goal = array([1,1])
        strategy = ConstNode(goal)
        
        my_agent.strategy_tree = strategy

        lt, game = get_mock_game(layout_txt, [my_agent])
        game._init_run()
        
        pos = game.state.getAgentPosition(0)
        for _ in xrange(10):
            game._one_game_step()
        # test that the agent really moved east!
        new_pos = array(game.state.getAgentPosition(0))
        print pos, new_pos

        assert (new_pos == goal).all()




        
    def test_case3(self):
        layout_txt = """%%%%%%%%%%%%%%%%%%%%
                        %..................%
                        %..................%
                        %..................%
                        %........1.........%
                        %..................%
                        %..................%
                        %..................%
                        %%%%%%%%%%%%%%%%%%%%"""

        
        my_agent = GPAgent(0,None)
        addw = my_agent.addw
        hasfoodw = my_agent.hasfoodw

        strategy = Node(hasfoodw, [
                        Node(addw, [ParamNode(0), ConstNode(array([0,1]))]),
                        Node(addw, [ParamNode(0), ConstNode(array([0,1]))]),
                         Node(
                         hasfoodw, [
                          Node(addw, [ParamNode(0), ConstNode(array([1,0]))]),
                          Node(addw, [ParamNode(0), ConstNode(array([1,0]))]),
                          Node(
                          hasfoodw, [
                           Node(addw, [ParamNode(0), ConstNode(array([-1,0]))]),
                           Node(addw, [ParamNode(0), ConstNode(array([-1,0]))]),
                           Node(
                           hasfoodw, [
                            Node(addw, [ParamNode(0), ConstNode(array([0,-1]))]),
                            Node(addw, [ParamNode(0), ConstNode(array([0,-1]))]),
                            Node(addw, [ParamNode(0), ConstNode(array([0,-1]))])
                            
                           ])
                          ]) 
                         ])
                        ])

        my_agent.strategy_tree = strategy

        lt, game = get_mock_game(layout_txt, [my_agent])
        game._init_run()
        for i in xrange(20):
            game._one_game_step()
            score = my_agent.get_score(game.state)
        print 'Score', score

        self.assertTrue(score>10)

if __name__ == '__main__':
    unittest.main()

