from random import random, randint, choice
from copy import deepcopy
from math import log

class FuncWrapper(object):
    def __init__(self, func, childcount, name):
        self.func = func
        self.childcount = childcount
        self.name = name


class Node(object):
    def __init__(self, fw, children):
        self.func = fw.func
        self.name = fw.name
        self.children = children

    def evaluate(self,inp):
        results = [n.evaluate(inp) for n in self.children]
        return self.func(results)
    
    def display(self, indent=0):
        print ' '*indent + self.name
        for c in self.children:
            c.display(indent+1)


class ParamNode(object):
    def __init__(self, idx):
        self.idx = idx

    def evaluate(self, inp):
        return inp[self.idx]

    def display(self, indent=0):
        print '%sp%d' % (' '*indent, self.idx)

class ConstNode(object):
    def __init__(self, v):
        self.v = v
    def evaluate(self, inp):
        return self.v
    def display(self, indent=0):
        print '%s%d' % (' '*indent, self.v)

addw = FuncWrapper(lambda l: l[0]+l[1], 2, 'add')
subw = FuncWrapper(lambda l: l[0]-l[1], 2,'subtract')
mulw = FuncWrapper(lambda l: l[0]*l[1], 2, 'multiply')

def iffunc(l):
    if l[0]>0: 
        return l[1]
    else:
        return l[2]
ifw = FuncWrapper(iffunc, 3, 'if')

def isgreater(l):
    if l[0]>l[1]: 
        return l[1]
    else:
        return 0
gtw = FuncWrapper(isgreater, 2, 'isgreater')

flist_basic = [addw, mulw, ifw, gtw, subw]

def makerandomtree(pc,  maxdepth=4, fpr=0.5, ppr=0.6, func_list=None):
    flist = None
    #here comes python kung-fu to create addtional namespace!!!
    def initialisedtree(pc, maxdepth=4, fpr=0.5, ppr=0.6, func_list=None):
        if random() < fpr and maxdepth>0:
            f = choice(flist)
            children = [makerandomtree(pc, maxdepth-1, fpr, ppr)
                    for i in range(f.childcount)]
            return Node(f, children)
        elif random() < ppr:
            return ParamNode(randint(0, pc-1))
        else:
            return ConstNode(randint(0, 10))

    
    if not flist:
        if not func_list:
            flist = flist_basic
        else:
            flist = func_list
        makerandomtree = initialisedtree
    return makerandomtree(pc, maxdepth, fpr, ppr)

def mutate(t, pc, prob=0.01):
    if random()<prob:
        return makerandomtree(pc)
    else:
        result = deepcopy(t)
        if isinstance(t, Node):
            result.children = [mutate(c, pc, prob) for c in
                    t.children]
        return result

def crossover(t1, t2, prob=0.7, top=1):
    if random()<prob and not top:
        return deepcopy(t2)
    else:
        result = deepcopy(t1)
        if isinstance(t1, Node) and isinstance(t2, Node):
            result.children = [crossover(c, choice(t2.children), prob, 0)
                               for c in t1.children]
        return result


def evolve(pc, popsize, rankfunction, maxgen=500, mutationrate=0.1,
        breedingrate=0.4, pexp=0.7, pnew=0.05, func_list=None):
    def selectindex():
        return int(log(random())/log(pexp))
    population = [makerandomtree(pc, func_list=func_list) for i in range(popsize)]
    for i in range(maxgen):
        scores = rankfunction(population)
        print scores[0][0]
        if scores[0][0] == 0: 
            break

        newpop = [scores[0][1], scores[1][1]]

        while len(newpop) < popsize:
            if random() > pnew:
                newpop.append(mutate(
                    crossover(scores[selectindex()][1],
                        scores[selectindex()][1],
                        prob=breedingrate),
                        pc, prob=mutationrate))
            else:
                newpop.append(makerandomtree(pc))
        population = newpop
    return scores[0][1]

def getrankfunction(dataset, scorefunction):
    def rankfunction(population):
        scores = [(scorefunction(t,dataset), t) for t in population]
        scores.sort()
        return scores
    return rankfunction

if __name__ == '__main__':
    random1 = makerandomtree(2)
    print random1.display()
    print "Result:", random1.evaluate([7, 1])


