import unittest

from gp import Node, ConstNode, ParamNode, ifw, addw, subw, gtw
import gp
from random import randint

class TreeTest(unittest.TestCase):
    
    def setUp(self):

        self.exampletree = Node(ifw, [
                                      Node(gtw, [ParamNode(0), ConstNode(3)]),
                                      Node(addw, [ParamNode(1), ConstNode(5)]),
                                      Node(subw, [ParamNode(1), ConstNode(2)]),
                                    ]
                              )

    def test_tree(self):
        """if x>3: y+5: else: y-2"""
        res1 = self.exampletree.evaluate([2,3])
        res2 = self.exampletree.evaluate([5,3])
        self.assertEqual(res1, 1)
        self.assertEqual(res2, 8)


def buildhiddenset(func, pc, range=(0,40), n=200):
    trials = []
    for i in xrange(n):
        params = tuple([randint(*range) for x in xrange(pc)])
        trials.append(params + (func(*params), ))
    return trials

def scorefunction(tree, s):
    dif = 0
    for data in s:
        v = tree.evaluate(data[:-1])
        dif += abs(v - data[-1])
    return dif

class GPTest(unittest.TestCase):

    def test_const(self):
        def hiddenfunction():
            return 5
        rf = gp.getrankfunction([5], hiddenfunction)
        best = gp.evolve(0, 500, rf, maxgen=2, mutationrate=0.2, breedingrate=0.1,
                pexp=0.7, pnew=0.1)
        result = best.evaluate([])
        self.assertAlmostEqual(result, hiddenfunction(), 2)
    
    def test_linear(self):
        def hiddenfunction(x):
            return 2*x
        x_test=5
        rf = gp.getrankfunction(buildhiddenset(hiddenfunction, 1),
                scorefunction)
        best = gp.evolve(1, 500, rf, maxgen=1, mutationrate=0.2, breedingrate=0.1,
                pexp=0.7, pnew=0.1)
        result = best.evaluate([x_test])
        print best.display()
        self.assertAlmostEqual(result, hiddenfunction(x_test), 2)
    
    def test_quadratic(self):
        def hiddenfunction(x):
            return (x-2)**2
        x_test=5
        rf = gp.getrankfunction(buildhiddenset(hiddenfunction, 1),
                scorefunction)
        best = gp.evolve(1, 500, rf, maxgen=100, mutationrate=0.1, breedingrate=0.1,
                pexp=0.7, pnew=0.1)
        result = best.evaluate([x_test])
        print best.display()
        self.assertAlmostEqual(result, hiddenfunction(x_test), 2)



if __name__ == '__main__':
    unittest.main()

