"""
Unittests for the graph module.
"""

import unittest
import numpy as np

from niko_agents import KickAssHQ


class TestBasicKickAssHQ(unittest.TestCase):
    
    def setUp(self):
        """Manually initialize a simple KickAssHQ instance."""
        self.hq = KickAssHQ()
        self.hq.wall_map = np.array([[False, False, False],
                                     [True, True, False],
                                     [False, False, False]])
        self.hq._map_indices = [(i,j)
                        for i in range(self.hq.wall_map.shape[0])
                        for j in range(self.hq.wall_map.shape[1])]
        
    def test_dist_map(self):
        """Test the distance calculation method."""
        dist_map = np.array([[0,-1,0],[-1,0,-1],[-1,-1,-1]])
        self.hq.calc_distance_map(dist_map)
        reference = np.array([[ 0,1,0],
                              [-1,0,1],
                              [ 4,3,2]])
        self.assert_(np.all(dist_map == reference))
        
    def test_dist_map_max(self):
        """Test the maximum distance argument of the calculation method."""
        dist_map = np.array([[0,-1,0],[-1,0,-1],[-1,-1,-1]])
        self.hq.calc_distance_map(dist_map, max_dist=2)
        reference = np.array([[ 0,1,0],
                              [-1,0,1],
                              [ -1,-1,2]])
        self.assert_(np.all(dist_map == reference))            
    
    
if __name__ == '__main__':
    unittest.main()