"""
KickAssAgents that say twitter messages with the #python hash.
"""

import simplejson
import urllib

from basic_agents import BasicAgentFactory, BasicAgent

import niko_agents

MAX_SAY = 20  # number of times the tweet is repeated
N_TWEETS = 100  # max is 100

            
class TwitterKickAssAgents(niko_agents.KickAssAgents):
    """Returns one offensive reflex agent and one defensive reflex agent"""
    
    def __init__(self, **args):
        """Initialize agent factory."""
        super(TwitterKickAssAgents, self).__init__(**args)
        self._agent_hq = TwitterKickAssHQ(niko_agents.DEFAULT_HQ_PARAMS)
        self.agents = []
 
    def getAgent(self, index):
        """Return a new agent."""
        # workaround for capture bug, it requests 3 agents
        if len(self.agents) >= 2:
            return BasicAgent(index)
        new_agent = TwitterKickAssAgent(index, self._agent_hq,
                                        niko_agents.DEFAULT_AGENT_PARAMS)
        self.agents.append(new_agent)
        return new_agent
    
    
class TwitterKickAssHQ(niko_agents.KickAssHQ):
    """KickAssHQ that downloads and stores the twitter messages."""
    
    def __init__(self, params):
        super(TwitterKickAssHQ, self).__init__(params)
        self.tweets = None  # list of tweets
        self.tweet_index = 0  # index of last used tweet
    
    def initialize_hq(self, game_state):
        """Initialization with the initial state, downlaid twitter messages."""
        super(TwitterKickAssHQ, self).initialize_hq(game_state)
        # get twitter messages on python
        url = ('http://search.twitter.com/search.json?q=python&rpp=%d' %
               N_TWEETS)
        result = simplejson.load(urllib.urlopen(url))
        self.tweets = result[u'results']
        
    
class TwitterKickAssAgent(niko_agents.KickAssAgent):
    """KickAssAgent that says the twitter messages from the agent hq."""
    
    def __init__(self, index, agent_hq, params):
        """Initialize the agent, see KickAssAgent."""
        super(TwitterKickAssAgent, self).__init__(index, agent_hq, params)
        self._say_counter = 100000  # counter for the_textt repetitions
        self._text = ""  # _text the agent says
    
    def choose_action(self, game_state):
        """Say a twitter message and then return the action from parent."""
        if self._say_counter >= MAX_SAY:
            self._say_counter = 0
            if self._agent_hq.tweet_index >= len(self._agent_hq.tweets):
                self._agent_hq.tweet_index = 0
            tweet = self._agent_hq.tweets[self._agent_hq.tweet_index]
            try:
                self._text = str(tweet[u'from_user'] + ": " + tweet[u'text'])
            except UnicodeEncodeError:
                self._text = "twitter: Can't touch this!"
            self._agent_hq.tweet_index += 1
        self._say_counter += 1
        self.say(self._text)
        return super(TwitterKickAssAgent, self).choose_action(game_state)
    