#!/usr/bin/env python
#coding=utf-8

from basic_agents import BasicAgent, BasicAgentFactory
import random
import util
import numpy
from game import Directions
from rike_paraphernalia import ShortestPath, MazeAnalyser
import rike_paraphernalia as paraphernalia

class RikeAgents(BasicAgentFactory):
    """Supernumeraries. The stars are in the Squad.
    """
    def get_agents_list(self):
        return [WallIsAlwaysRightAgent, BrownianAgent]
        #return [SomeYetUnnamedAgent, WallIsAlwaysRightAgent]
        #return [WallIsAlwaysRightAgent, PongAgent]
    
class RikeAgentSquad(BasicAgentFactory):
    def __init__(self, **args):
        msg = """The RikeAgentSquad is now called the TheAgentSquad.
Please find our new business cards right at the entrance or ask
our caring and friendly staff members for an update.
         
        """
        raise NameError(msg)

class TheAgentSquad(BasicAgentFactory):
    """
    The TheAgentSquad is a secretly operating assassination squad and is
    specialised on mazes.
    Taking their power from a hidden plant growing in the Amazonian rain forest,
    the squad members are virtually unbeaten. Well-known underground
    organisations from several countries from both developing countries and
    highly industrialised nations all over the world have one thing in common: 
    They fear the TheAgentSquad!
    
    Consisting of many tough guys and a 30 % quota of ever such tough women
    (the quota is rapidly increasing – as is the quota of men, smarty), all of
    the squad members have gone through five years on most intensive
    instruction and education in martial arts and film quotes. Their abilities
    are beyond compare.
    
    It is rumoured that some of our mentors even know the secret of levitation,
    but it is a fact that gravity seems to have a much lesser influence on them
    than on other people.
    """
    def __init__(self, **args):
        super(TheAgentSquad, self).__init__(**args)
        self._agents_list = self.get_agents_list()
        self._agent_idx = 0
        self.hq = MI0() 
        
    def get_agents_list(self):
        return [SomeYetUnnamedAgent, SomeYetUnnamedAgent]

    def getAgent(self, index):
        "Returns the agent for the provided index."
        idx = self._agent_idx
        if idx >= len(self._agents_list):
            return BasicAgent(index)        
        agent_class = self._agents_list[self._agent_idx]
        self._agent_idx += 1
        agent = agent_class(index)
        agent.hq = self.hq
        self.hq.enlist_agent(agent)
        return agent

class MI0(object):
    """
    Located beyond ground level, this beautifully designed palace of
    marble and stainless steel serves as the headquarters for the
    RikeAgentSquad.
    
    Ever so often patrons and godfathers from a great many of infamous
    families and organisations are seen sitting on the porch of MI0,
    humiliated and waiting for remission.
    
    But the MI0 is not as unhuman as you might think. Inside there are
    a plenitude of rooms for recreation. Bars, pools, clubs, an ice skating
    area, a mountain luge, a cart track, you name it. Why? Because we care.
    
    Of course, the headquarters serve also for the strategic planning of 
    outdoor activities. Completely wired, the heart of MI0 has access
    to one of the greatest supercomputers and dozens of smart brains
    educated at our own university. So smart in fact, that in their spare
    time they will do cancer research and fold molecules.
    """
    def __init__(self):
        # TODO: make stuff protected
        self.agents_can_die = True
        self.agents = {}
        self.agent_codes = {}
                
        self.food_protect = []
        self.food_protect_prev = []
        
        self.enemies = {}
        self.enemies['history'] = []
        self.time = 0
        
        # not the best style to put those here. but they need to be central
        self.shortest_path = None
        self.maze_analyser = None
        
        self.strategy = None # for the overall strategy
        self.agent_tactics = [] # the individual tactics
        
        # Brainstorming:
        # Checkmate (occupying a space before a dead end)
        # 
        
        self.registered = False
        self.food_begin = None # Amount of food for enemy.
        
        # The implementation of the talking module is done in an
        # external class. Some composition is used to access it.
        self.talking_module = paraphernalia.AgentTalk()
        self.select_talk = self.talking_module.select_talk # method
        self.talk = self.talking_module.talk # method
        self.has_talk = self.talking_module.has_talk # method
        self.talk_topics = self.talking_module.topics # dict
        
    def register_game(self, agent, game_state):
        if self.registered: return
        self.food_protect = numpy.array(agent.get_food(game_state, enemy_food=False).data, 'bool')
        self.food_begin = numpy.sum(self.food_protect)
        
        self.registered = True
    
    def step(self, agent, game_state):
        """Saves a few important things each step.
        """
        self.time += 0.5
        
        self.food_protect_prev = self.food_protect
        self.food_protect = numpy.array(agent.get_food(game_state, enemy_food=False).data, 'bool')
        
        pos = self._trace_enemies_by_eaten_food()

        positions = agent.get_team_positions(game_state, enemy_pos=True)
        while pos:
            p = pos.pop()
            if not p in positions:
                if None in positions:
                    positions[positions.index(None)] = p
                else:
                    positions.append(p)
        
        #print positions 
        self.enemies['history'].append(positions)
        
    def _trace_enemies_by_eaten_food(self):
        """
        Finds the enemy position by tracing the eaten food. Clever, eh?
        """
        pos = []
        if self.food_protect_prev != []:
            diff = numpy.logical_xor(self.food_protect, self.food_protect_prev)
            diff_points = numpy.nonzero(diff)
            if diff_points:
                x = diff_points[0]
                y = diff_points[1]
                for i in range(x.size):
                    pos.append( (x[i], y[i]) )
        return pos
    
    def enlist_agent(self, agent):
        specialities = AgentSpecialities()
        self.agents[agent.index] = specialities
        print "We welcome our new 00%s agent, %s." % (agent.index, specialities.agent_name)
        # print specialities
    
    def agent(self, index):
        return self.agents[index]
    
    def specialities(self, agent):
        return self.agents[agent.index]
    
    def take_upper(self, agent):
        return self.best_agent("takes_upper") == agent.index
    
    def has_more_experience(self, agent):
        return self.best_agent("experience") == agent.index
    
    def agent_name(self, agent):
        return self.agents[agent.index].agent_name
    
    def best_agent(self, speciality):
        return max(self.agents, key=lambda ai: getattr(self.agents[ai], speciality))
    
    @property
    def index_master(self):
        index = self.best_agent("experience")
        return index
    
    @property
    def master(self):
        index = self.index_master
        return self.agents[index]
    
    @property
    def index_rand_slave(self):
        all_slaves = self.index_all_slaves
        return random.choice(all_slaves)
    
    @property
    def rand_slave(self):
        index = self.index_rand_slave
        return self.agents[index]
    
    @property
    def index_all_slaves(self):
        all_slaves = self.agents.keys()
        all_slaves.remove(self.index_master)
        return all_slaves
    
    def index_all_other(self, agent):
        all_other = self.agents.keys()
        all_other.remove(agent.index)
        return all_other
    
    def index_rand_other(self, agent):
        all_other = self.index_all_other(agent)
        return random.choice(all_other)
    
    AGENT_CODES = ["00" + str(n) for n in range(9)]


class AgentSpecialities():
    """Our Agents have a vast range of different abilities and are in fact
    specialised on certain techniques. Nonetheless, if history desires, they
    will go even beyond their trademark styles and do whatever is pleased
    and needed.
    """
    
    
    AGENT_NAMES = ["Fu Manchu", "Agent 86", "Agent 99", "James Bond, Jr.",
                   "John Steed", "Dr David Keel", "Cathy Gale", "Emma Peel",
                   "Tara King", "Danny Wilde", "Lord Brett Sinclair",
                   "Voice On Tape", "Inspector Gadget", "Austin Powers",
                   "Dr. Evil", "Vesper Lynd", "Jaws", "Modesty Blaise"]
    def __init__(self):
        self.goes_for = []
        self.takes_upper = random.random()
        self.experience = random.random()
        self.agent_name = random.choice(self.AGENT_NAMES)
        self.health = random.random()
        self.protecting = []
        
    def __repr__(self):
        return {'goes_for': self.goes_for,
                'takes_upper': self.takes_upper,
                'experience': self.experience,
                'agent_name': self.agent_name,
                'health' : self.health,
                'protecting' : self.protecting
                }.__repr__()

class ProgressiveAgent(BasicAgent):
    """
    Slightly advanced agent, with some more functions available.
    """
    def __init__(self, index):
        super(ProgressiveAgent, self).__init__(index)
        self.maze = None
        self.maze_len_x = 0 # width
        self.maze_len_y = 0 # height
        self.lifetime_counter = 0
        self.position_history = []

    def register_initial_state(self, game_state):
        super(ProgressiveAgent, self).register_initial_state(game_state)
        
        # This should not be done in the agent’s code, but well.
        # At least one can argue, that it’s convenient that way.
        self.maze = game_state.getWalls().data
        self.maze = numpy.array(self.maze, dtype='bool')#.T

        self.maze_len_x = self.maze.shape[0]
        self.maze_len_y = self.maze.shape[1]
    
    def is_on_our_side(self, pos):
        """Return true if x is on allied side of the maze."""
        half = self.maze_len_x // 2
        if self.is_red:
            return pos[0] < half
        else:
            return pos[0] >= half
    
    def is_just_on_our_side(self, pos):
        """Return true if x is on allied side of the maze."""
        half = self.maze_len_x // 2
        
        if self.is_red:
            return pos[0] == half - 1
        else:
            return pos[0] == half

    def safe_points(self, game_state):
        """
        Places to reach to be on the safe side.
        """
        width = self.maze_len_x
        height = self.maze_len_y
        if self.is_red:
            possible_points = [(width // 2 - 1, p) for p in range(height)]
        else:
            possible_points = [(width // 2, p) for p in range(height)]
        safe_points = [p for p in possible_points if not game_state.hasWall(p[0], p[1])]
        return safe_points

    def is_killed(self, game_state):
        """Return True if the agent has been eaten on the previous turn."""
        start_position = game_state.getInitialAgentPosition(self.index)
        return game_state.getAgentPosition(self.index) == start_position
    
    def get_food_coordinates(self, game_state, enemy_food=True): # from pietro
        """Return the maze coordinates of food belonging to one of the teams.

        Keyword arguments:
        enemy_food -- If True, returns the food you're supposed to eat,
                    otherwise return the food you should protect.
                    (default: True)
        """
        food = self.get_food(game_state, enemy_food=enemy_food)
        food = numpy.array(food.data, 'bool')
        idx = food.nonzero()
        return zip(idx[0], idx[1])
        
    def choose_action(self, game_state):
        """Increases the lifetime_counter. Should be called with super()
        """
        self.lifetime_counter += 1
        self.position_history.append(self.get_position(game_state))
        return Directions.STOP
    
    def get_position(self, game_state):
        """Returns the current position of the agent.
        """
        return game_state.getAgentState(self.index).getPosition()
    
    def sill_has_to_eat(self, game_state, enemy_food=True):
        """Returns the amount of food that still has to be eaten.
        """
        return numpy.sum(numpy.array(self.get_food(game_state, enemy_food).data))
    
    @property
    def pos_now(self):
        try:
            return self.position_history[-1]
        except IndexError:
            return None
    
    @property
    def pos_prev(self):
        try:
            return self.position_history[-2]
        except IndexError:
            return None

class SpecialAgent(ProgressiveAgent):
    """
    Defines some common things for special agents, belonging to the Squad.
    """
    def __init__(self, index):
        super(SpecialAgent, self).__init__(index)
        self.hq = None
        
    @property
    def is_master(self):
        return self.hq.index_master == self.index
    
    @property
    def is_slave(self):
        return not self.is_master
    
    @property
    def specialities(self):
        return self.hq.specialities(self)
    
    def choose_action(self, game_state):
        """Increases the lifetime_counter. Should be called with super()
        """
        super(SpecialAgent, self).choose_action(game_state)
        self.hq.step(self, game_state)
        
        if self.hq and self.hq.agents_can_die and self.is_killed(game_state):
            hit_strength = random.random()
            if self.specialities.health > hit_strength:
                print "%s has been wounded but survived the attack. 00%s agent continues their mission." % (self.specialities.agent_name, self.index) 
            else:
                print "%s has died after %s steps in game. Sending a new 00%s agent on the mission." % (self.specialities.agent_name, self.lifetime_counter, self.index)
                self.lifetime_counter = 0
                self.hq.enlist_agent(self)
    
    def be_bright_or_be_quiet(self):
        """Says something, if there is something to say"""
        talk = self.hq.talk(self.index)
        if talk:
            self.say(talk)
        else:
            self.say('')


class SomeYetUnnamedAgent(SpecialAgent):
    def __init__(self, index):
        super(SomeYetUnnamedAgent, self).__init__(index)
        
        self.border_movement_path = []
        
    def register_initial_state(self, game_state):
        super(SomeYetUnnamedAgent, self).register_initial_state(game_state)

        if not self.hq.maze_analyser:
            self.hq.maze_analyser = MazeAnalyser(game_state.getWalls().data)
        if not self.hq.shortest_path: 
            self.hq.shortest_path = ShortestPath(self.hq.maze_analyser)        
        
        self.say(self.hq.agent_name(self))
        self.tactic = {}
        self.tactic['loop_detection'] = LoopDetection(self.hq, self, game_state)
    

    def is_killed(self, game_state):
        """Return True if the agent has been eaten on the previous turn.
        This version takes into account, that one might walk accidently
        or intendedly to the initial position.
        """
        return super(SomeYetUnnamedAgent, self).is_killed(game_state) and \
            self.pos_now and self.pos_prev and \
            self.hq.maze_analyser.path_length(self.pos_now, self.pos_prev) > 1

    def split_food(self, food_orig, direction):
        """Returns the food in a certain direction.
        Direction in ['n', 's', 'e', 'w']
        """
        if direction == 'n':
            food_part = [food for food in food_orig if food[1] >= self.maze_len_y//2]
        elif direction == 's':
            food_part = [food for food in food_orig if food[1] < self.maze_len_y//2]
        elif direction == 'w':
            part_x = 1 if self.is_red else 3
            food_part = [food for food in food_orig if food[0] >= part_x*self.maze_len_x//4]
        elif direction == 'e':
            part_x = 1 if self.is_red else 3
            food_part = [food for food in food_orig if food[0] < part_x*self.maze_len_x//4]
        return food_part

    def get_best_splitting(self, food, game_state):
        """Returns true, if north–south splitting
        """
        #TODO: my_pos should be done for both!!!
        my_pos = self.get_position(game_state)
        
        food_length = {}
        directions = ['n', 's', 'e', 'w']
        
        for d in directions:
            food_part = self.split_food(food, d)
            p = self.hq.shortest_path.solve_TSP(my_pos, food_part)
            food_length[d] = self.hq.shortest_path.length_of_path(p)
        
        return food_length['n'] + food_length['s'] <= food_length['e'] + food_length['w']
    
    
    def choose_action(self, game_state):
        super(SomeYetUnnamedAgent, self).choose_action(game_state)
        
        #if self.is_slave and self.lifetime_counter < 5:
        #        return Directions.STOP
        
        if self.get_score(game_state) - self.sill_has_to_eat(game_state, True) > 0:
            self.say("We win!")
        if self.get_score(game_state) - self.sill_has_to_eat(game_state, True) < 0 and \
            self.get_score(game_state) > 5:
            pass
        # defense_mode
        
        
        my_pos = self.get_position(game_state)
        
        food_c = self.get_food_coordinates(game_state, True)
        
        north_south_splitting = True#self.get_best_splitting(food_c, game_state)
        
        if self.hq.take_upper(self) and north_south_splitting:
            direction = 'n'
        if not self.hq.take_upper(self) and north_south_splitting:
            direction = 's'
        if self.hq.take_upper(self) and not north_south_splitting:
            direction = 'e'
        if not self.hq.take_upper(self) and not north_south_splitting:
            direction = 'w'
            
        upper_food = self.split_food(food_c, direction)
        
        if not upper_food:
            upper_food = food_c
            
        best_path = self.hq.shortest_path.solve_TSP(my_pos, upper_food)
        best_path = [best_path[0]] + self.hq.shortest_path.solve_TSP(best_path[0], food_c)
        
        enemies = self.hq.enemies['history'][-1]
        for e in enemies:
            if e is None or self.get_score(game_state) < 0: continue
            d_e = self.hq.maze_analyser.in_dead_end(game_state, e)
            if d_e and d_e[0] in self.hq.maze_analyser.before_dead_end(game_state, self.get_position(game_state)):
                if self.get_position(game_state) != game_state.getAgentState(self.hq.index_rand_other(self)).getPosition():
                    self.hq.specialities(self).protecting = d_e
                    # only wait, when noone else is waiting here
                    return Directions.STOP
        
        
        #TODO: if waiting on border and standing still: move vertically
               
        # If enemies are close, try to fight them early 
        enemies = self.hq.enemies['history'][-1]
        enemies_tracked = False
        for e in enemies:
            if e:
                self.tactic['loop_detection'].has_loop(self.hq, e, min_loops=2, max_lag=20)
            
            if e is not None and self.is_on_our_side(e):
                protected_positions = [self.hq.agent(i).protecting for i in self.hq.index_all_other(self)]
                if not e in protected_positions:
                    best_path.insert(0, e)
                    enemies_tracked = True
                    

        if enemies_tracked:
            best_path = self.hq.shortest_path.solve_TSP(my_pos, best_path[0:5])
            self.say("Enemy alarm!")
        else:
            pass
            #self.say(random.choice(["Foood!", "Hungry!"]))
        
        enemies_tracked = False

        for e in enemies:
            if e is not None and (not self.is_on_our_side(e) or my_pos in self.safe_points(game_state)):
                enemies_tracked = True        
        
        self.fear_distance = 4
        for dist, pos in self.distance_to_enemies(game_state) :
            if dist < self.fear_distance and (not self.is_on_our_side(pos)):# or my_pos not in self.safe_points(game_state)):
                best_path = self.fear_path(game_state)
                self.say("Enemy alarm!")
                break

        if len(best_path):
            best_pos = best_path[0]
            
            if self.hq and self.is_slave:
                if len(self.hq.master.goes_for) > 1:
                    if best_pos == self.hq.master.goes_for[0]:
                        best_path.remove(best_pos)
                        best_path = self.hq.shortest_path.solve_TSP(my_pos, best_path[0:5])
        
        
        if random.random() > 0.95:
            self.hq.select_talk('Hunger', self.index, self.hq.index_rand_other(self))
        
        if self.lifetime_counter % 20 == 0:
            if self.is_master:
                self.hq.select_talk(random.choice(self.hq.talk_topics), self.index, self.hq.index_rand_slave)
        self.be_bright_or_be_quiet()

        if self.lifetime_counter <= 5:
            self.say(self.hq.agent_name(self))

        self.path = best_path
        if self.hq:
            self.hq.master.goes_for = self.path
        
        try:
            way = self.hq.maze_analyser.get_shortest_path(my_pos, best_path[0] )[0][0]
            #if self.is_just_on_our_side(my_pos):
            #    p = self.border_movement(game_state)
            #    print p
            #    print self.hq.maze_analyser.get_shortest_path(my_pos, p)
            #    way = self.hq.maze_analyser.get_shortest_path(my_pos, p)[0][0]
        
        except IndexError:
            way = Directions.STOP
        
        
        
        if way in game_state.getLegalActions(self.index):
            return way
        return Directions.STOP
    
    def distance_to_enemies(self, game_state):
        """
        Returns a tuple of (distance, positions)
        """
        
        if self.hq.enemies:
            positions = self.hq.enemies['history'][-1]
        else:
            positions = self.get_team_positions(game_state, enemy_pos=True)
        return [(self.hq.maze_analyser.path_length(self.get_position(game_state), p), p) for p in positions if p]
    
    def fear_path(self, game_state):
        possible_points = self.safe_points(game_state) 
        my_pos = self.get_position(game_state)
        
        # take the best food
        food_c = self.get_food_coordinates(game_state, enemy_food=True)
        food_c = [f for f in food_c if not self.hq.maze_analyser.in_dead_end(game_state, f)]
        
        enemy_pos = self.hq.enemies['history'][-1]
        enemy_pos = [e for e in enemy_pos if e]
        
        possible_points_distances = map(lambda x: (x, self.hq.maze_analyser.path_length(x, my_pos)), possible_points + food_c)
        possible_points_distances.sort(key=lambda x: x[1])
        
        enemies = self.get_team_positions(game_state)
        enemy_path = filter(lambda x: x, enemies)
        
        path_to_take = None
        for pos in possible_points_distances:
            print enemy_pos, pos[0]
            path = self.hq.shortest_path.get_shortest_path_for_agent(self, pos[0], game_state, fear_factor=10, avoid=enemy_pos)
            enemy_in_path = False
            for e in enemy_path:
                if e in path: enemy_in_path = True
            if not enemy_in_path:
                path_to_take = path
                break
 
        path_to_take = map(lambda x: x[1], path_to_take)  
        return path_to_take


class Strategy(object):
    pass

class Tactic(object):
    def __init__(self, hq, agent, game_state):
        pass
    def pre_condition(self, hq, agent, game_state):
        pass
    def movement(self, hq, agent, game_state):
        pass


class LoopDetection(Tactic):
    """Detect a loop and, if winning, keep repeating the same movements.
    If losing. Break the loop.
    """
    def has_loop(self, hq, enemy_pos, min_loops=2, max_lag=20):
        #print "pos", enemy_pos, hq.enemies['history'][-1]
        enemy_movements = hq.enemies['history']
        enemy_movements.reverse()
        if not enemy_pos in enemy_movements[0]: return False
        neighbours = []
        possible_movements = []
        last_neighbours = [enemy_pos]
        for i in range(max_lag):
            for enemy_pos in last_neighbours:
                #print "movem", enemy_pos, enemy_movements[i]
                #print "neighb", paraphernalia.find_neighbour_of_in(enemy_pos, enemy_movements[i], include_self=True)
                neighbours.append(paraphernalia.find_neighbour_of_in(enemy_pos, enemy_movements[i], include_self=True))
            last_neighbours = neighbours[-1]
        # now the neighbours object includes all possible positions that neigbour one previous position
        #print neighbours
        #for moves in enemy_movements:
            
        
    def pre_condition(self, hq, agent, game_state):
        pass
    
    
class BorderPatrol(Tactic):
    def movement(self, hq, agent, game_state):
        my_pos = agent.get_position(game_state)
        if agent.border_movement_path:
            p = agent.border_movement_path[0]
            while True:
                p = agent.border_movement_path.pop()
                agent.border_movement_path.insert(0, p)
                if agent.hq.maze_analyser.path_length(p, my_pos) <= 2:
                    return agent.border_movement_path[0]
            
        safe = agent.safe_points(game_state)
        print safe
        agent.border_movement_path = sorted(safe) + \
                        sorted(safe, reverse=True)
        
        p = agent.border_movement_path[0]
        while p != my_pos:
            p = agent.border_movement_path.pop()
            agent.border_movement_path.insert(0, p)
        return p 
        
        

class ReflexiveSpecialAgent(SpecialAgent):
    """
    A base class for reflex agents that chooses score-maximizing actions
    """
    def choose_action(self, game_state):
        """
        Picks among the actions with the highest Q(s,a).
        """
        super(ReflexiveSpecialAgent, self).choose_action(game_state)
        actions = game_state.getLegalActions(self.index)
        
        self.path_calculation(game_state)
        
        # You can profile your evaluation time by uncommenting these lines
        # start = time.time()
        values = [self.evaluate(game_state, a) for a in actions]
        # print 'eval time for agent %d: %.4f' % (self.index, time.time() - start)
        max_value = max(values)
        best_actions = [a for a, v in zip(actions, values) if v == max_value]
    
        return random.choice(best_actions)

    def get_successor(self, game_state, action):
        """
        Finds the next successor which is a grid position (location tuple).
        """
        successor = game_state.generateSuccessor(self.index, action)
        pos = successor.getAgentState(self.index).getPosition()
        if pos != util.nearestPoint(pos): 
          # Only half a grid position was covered
            return successor.generateSuccessor(self.index, action)
        else:
            return successor

    def evaluate(self, game_state, action):
        """
        Computes a linear combination of features and feature weights
        """
        features = self.get_features(game_state, action)
        weights = self.get_weights(game_state, action)
        return features * weights

    def path_calculation(self, game_state):
        pass
    
    def get_features(self, game_state, action):
        """
        Returns a counter of features for the state
        """
        features = util.Counter()
        successor = self.get_successor(game_state, action)
        features['successorScore'] = self.get_score(successor)
        return features

    def get_weights(self, game_state, action):
        """
        Normally, weights do not depend on the game_state.  They can be either
        a counter or a dictionary.
        """
        return {'successorScore': 1.0}

class WallIsAlwaysRightAgent(BasicAgent):
    """
    Walks as such that the wall is on its right side
    """
    def choose_action(self, game_state):
        

        actions = game_state.getLegalActions(self.index)
        try:
            i = self.order.index(self.lastAction)
        except ValueError:
            i = 0
        for j in range(4):
            k = (i + j - 1) % 4
            if self.order[k] in actions:
                self.lastAction = self.order[k]
                return self.lastAction
        self.lastAction = Directions.STOP

    def register_initial_state(self, game_state):
        super(WallIsAlwaysRightAgent, self).register_initial_state(game_state)
        
        self.lastAction = None
        self.order = [Directions.NORTH, Directions.WEST, Directions.SOUTH, Directions.EAST]

class BrownianAgent(BasicAgent):
    def __init__(self, index):
        super(BrownianAgent, self).__init__(index)
        self.index = index
        
    def getAction(self, state):
        actions = state.getLegalActions(self.index)
        self.say("Hi there!")
        return random.choice(actions)


