#!/usr/bin/env python
#coding=utf-8

from captureAgents import AgentFactory
from basic_agents import BasicAgent
import random
from game import Directions
import utils_val as uv
import MouseAgents as ma

Jonin = 'Jonin'
Anbu = 'Anbu'
Genin = 'Genin'
Choji = 'Choji'

class NinjaPlatoon(AgentFactory):

    def __init__(self, **args):
        AgentFactory.__init__(self, **args)
        self.village = NinjaVillage()
        self.debug = False

    def create_ninja(self, index, rank):
        return Ninja(index, self.village, debug=self.debug, rank=rank)

    def getAgent(self, index):
        if index < 2:
            return self.create_ninja(index, Jonin)
        elif index < 4:
            return self.create_ninja(index, Anbu)
        else:
            return self.create_ninja(index, Genin)

class JoninPlatoon(NinjaPlatoon):

    def getAgent(self, index):
        return self.create_ninja(index, Jonin)

class AnbuPlatoon(AgentFactory):

    def getAgent(self, index):
        return self.create_ninja(index, Anbu)

class SwissKiller(NinjaPlatoon):

    def getAgent(self, index):
        if index < 2:
            return self.create_ninja(index, Jonin)
        elif index < 4:
            return self.create_ninja(index, Choji)
        else:
            return self.create_ninja(index, Genin)

class NinjaVillage():
    def __init__(self):
        self.ninjas = []

    def recruit(self, ninja):
        self.ninjas.append(ninja)

    # FIXME the selection is really foobared 
    # if prospective food is much closer to us, than to our partners
    # but our partner is already hunting it
    # we should select that food
    # if on the other hand prospective food is much further from us than from
    # our partner, we should not select that food
    def team_mate_food(self, index):
        """ get food any team mates are tracking as a list of tuples """
        return [ninja.state.path.end for ninja in self.ninjas
                if ninja.index is not index
                and 'state' in dir(ninja)
                and ninja.state.state_name == "HungryState"
                and ninja.state.path is not None]

class NinjaState(object):
    def __init__(self, agent):
        self.agent = agent
        self.state_name = self.get_state_name()
        self.print_debug("entered")
        self.agent.say(self.state_name)

    def pre_state_change(self):
        """ check for state changes before choosing action """
        pass

    def choose_action(self):
        """ get the next action for this state """
        pass

    def post_state_change(self):
        """ check for state changes after choosing action """
        pass

    def get_state_name(self):
        s = str(self.__class__)
        return s[s.find('.')+1:s.rfind("'")]

    def print_debug(self, message):
        self.agent.print_debug(self.state_name+" -> "+message)

class FleeState(NinjaState):
    """ run away to a safe location """
    def __init__(self, agent, path=None):
        NinjaState.__init__(self, agent)
        self.halt_counter = self.agent.halt
        self.moving_to_flee = False

        self.select_flee_path()

    def pre_state_change(self):

        if self.agent.is_pacman:
            if not self.agent.ghost_is_close() and not self.moving_to_flee:
                self.print_debug("no more ghosts in sight")
                self.agent.reset_state()
                return True

        if self.agent.is_ghost:
            if self.agent.pacman_is_close():
                self.print_debug("pacman detected, engage pursuit")
                self.agent.state = PursuitState(self.agent)
            if not self.agent.ghost_is_close() and not self.moving_to_flee:
                self.print_debug("no more ghosts around, reset")
                self.agent.reset_state()
                return True
            if self.moving_to_flee and self.agent.position == self.agent.flee_point:
                self.print_debug("arrived at flee point")
                self.moving_to_flee = False
                self.agent.reset_state()
                return True

    def choose_action(self):
        # if the halt counter is at 0, meaning we stood still for the halt time
        # we check that we are not at the flee point, and move there if this is
        # so
        if self.halt_counter is 0:
            self.halt_counter = self.agent.halt
            if self.agent.is_ghost and self.agent.position != self.agent.flee_point:
                    self.print_debug("moving to flee_point")
                    self.path = self.agent.graph.bfs(self.agent.position,
                                self.agent.flee_point, seen=self.agent.opp_nodes)
                    self.moving_to_flee = True

        # flee state is a reactive state, enemy positions may change
        # hence we recompute our optimal flee path every turn
        # if ghots were no longer close a state change would have happend
        # we only do not compute a new path if we are moving to a flee point
        if not self.moving_to_flee:
            self.select_flee_path()

        #  no path found fall through to STOP
        if self.path is None:
            self.print_debug("path is None")
            pass
        # otherwise there are actions to take
        elif len(self.path) is not 0:
            self.print_debug("taking a step along the path")
            return self.path.pop()[0]
        # if there are no more actions decrement the halt counter
        elif self.halt_counter > 0:
            self.print_debug("decrement halt counter")
            self.halt_counter -= 1
        # we could not find any appropriate actions, freeze
        self.print_debug("Stopping!")
        return Directions.STOP

    def select_flee_path(self):
        self.print_debug("selecting new path")
        # plot a course to the closest border node, not leading through
        # any nodes within running distance of a ghost
        avoid_nodes = self.agent.nodes_close_to_ghosts()
        paths = self.agent.graph.bfs_all(self.agent.position,
                    self.agent.border_nodes,
                    seen=avoid_nodes)

        # in case we can't find any path, we try and plot on that doesn't go
        # through any nodes that enemies are currently on, and hope to get lucky
        if len(paths) is 0:
            self.print_debug("no paths that go closer to enemy found")
            paths = self.agent.graph.bfs_all(self.agent.position,
                    self.agent.border_nodes,
                    seen=self.agent.ghost_positions())
        # if there are still no paths we are really unlucky, and will probably
        # die soon
        if len(paths) is 0:
            self.print_debug("no escape path found!")
            self.path = None
            # FIXME here ideally we should see if there is any food that we can
            # get a hold of
            # because: this means we are locked in a tunnel with ghosts at
            # either side, but there may be some more food in this dead-end
            # consider making HungryState have a no_fear attribute
            # that just goes for food, regardless of where the enemy is
        # if we found paths we take the shortest
        else:
            self.path = paths[0]
            self.print_debug("escape path found to: "+str(self.path.end))

class HungryState(NinjaState):
    """ get the next closest food and plan to get there """
    def __init__(self, agent, path=None):
        NinjaState.__init__(self, agent)

        if path is None:
            # FIXME maybe sometimes we should choose random food
            # also if we initialise somewhere else, we should also not plot
            # course that leads us closer to enemy
            # FIXME also select a path here instead
            #self.path = self.agent.closest_target(self.agent.food_list)
            self.select_new_path()
        else:
            self.path = path

    def pre_state_change(self):

        if self.agent.is_pacman:
            if self.agent.ghost_is_real_close() and len(self.agent.food_list) < 3\
            and self.path is not None and not self.agent.ghost_in_path(self.path):
                self.print_debug("so close to victory, keep going")
                #what if the current path moves us closer to a certain position
                # for example an enemy ghost
                return False

            #elif self.agent.action_stack[-self.agent.halt:] == self.agent.halt*[Directions.STOP]:
            #    self.agent.set_Choji()
            #    self.agent.reset_state()
            #    return True

            elif self.agent.ghost_is_real_close():
                self.print_debug("ghost is real close, RUN!")
                self.agent.state = FleeState(self.agent)
                return True

        elif self.agent.is_ghost:
            if self.agent.pacman_is_close() and not \
            any([pos in self.agent.border_nodes \
            for pos in  self.agent.pacmen_positions()]):
                self.print_debug("pacman is close, engage pursuit")
                self.agent.state = PursuitState(self.agent)
                return True
            elif self.agent.ghost_is_real_close():
                # this can happen when we are close to the border, and there is
                # a ghost on the other side
                self.print_debug("ghost on other side")
                self.agent.state = FleeState(self.agent)
                return True

    def choose_action(self):
        # we plot a new course, if the food no longer exists
        # or if there is an enemy in the path, or close
        if self.path is not None:
            if self.path.end not in self.agent.food_list \
                or self.agent.ghost_in_path(self.path) \
                    or self.agent.ghost_is_close():
                self.select_new_path()
        else:
            self.select_new_path()
        # if a path was found we take it
        if self.path is not None and len(self.path) is not 0:
            self.print_debug("food"+str(self.path.end))
            return self.path.pop()[0]
        # otherwise we just stop right
        else:
            return Directions.STOP

    def select_new_path(self):
        self.print_debug("selecting new path")
        # select all closest paths to food, that do not lead through
        # nodes that are within running distance of ghosts
        avoid_nodes = self.agent.nodes_close_to_ghosts()
        paths = self.agent.graph.bfs_all(self.agent.position,
                    self.agent.food_list,
                    seen=avoid_nodes)
        # remove all paths that bring us to within run distance of existing
        # ghosts
        #paths = [p for p in paths if not 
        #        p.gets_closer_to(self.agent.ghost_positions(), self.agent.graph,
        #            self.agent.run_distance)]

        if len(paths) is 0:
            self.path = None
            # this could mean a few things, for example that we are locked in,
            # but not within run range, and there is no food left
            # or it could mean that we can't find anymore food
        else:
            team_food = self.agent.village.team_mate_food(self.agent.index)
            if len(team_food) is not 0:
                self.print_debug("team food detected: "+str(team_food))
                #first try closest
                if paths[0].end not in team_food:
                    self.print_debug("closest not yet taken")
                    self.path = paths[0]
                #then try 2nd next closest
                elif paths[-1].end not in team_food:
                    self.print_debug("furthest not yet taken")
                    self.path = paths[-1]
                # oethwise go for one of the left over ones randomly
                elif len(paths) > 2:
                    self.print_debug("selecting random path")
                    self.path = random.choice(paths[1:-1])
                else:
                    self.print_debug("all team food taken, selecting same path")
                    self.path = paths[0]
            else:
                self.path = paths[0]

        if self.path is None:
            self.print_debug("no food path found!")
        else:
            self.print_debug("found food path: "+str(self.path.end))

class HunterState(NinjaState):
    """ patrol our territory and hunt enemies """
    def __init__(self, agent, path=None):
        NinjaState.__init__(self, agent)
        if path is None:
            self.select_random_path()
        else:
            self.path=path

    def pre_state_change(self):
        if self.agent.pacman_is_close():
            self.print_debug("pacman is close, engage pursuit")
            self.agent.state = PursuitState(self.agent)
            return True

    def choose_action(self):
        if self.agent.food_was_eaten:
            self.print_debug("food was eaten, plotting course to get there")
            pos = self.agent.eaten_food[0]
            self.path = self.agent.graph.bfs(self.agent.position, pos,
                    seen=self.agent.opp_nodes)
        elif len(self.path) is 0:
            self.print_debug("arrived at protect spot, plotting new course")
            self.select_random_path()

        if self.path is not None and len(self.path) is not 0:
            return self.path.pop()[0]
        else:
            return Directions.STOP

    def select_random_path(self):
        paths = self.agent.graph.bfs_all(self.agent.position,
            self.agent.protect_list)
        if len(paths) is not 0:
            self.print_debug("found random target to protect")
            self.path = random.choice(paths)
        else:
            self.print_debug("found nothing to protect, apparently")

class PursuitState(NinjaState):
    """ Enemy sighted , engage in pursuit """
    def __init__(self, agent):
        NinjaState.__init__(self, agent)
        #FIXME need to remember which agent we are tracking

    def pre_state_change(self):
        if not self.agent.pacman_is_close():
            self.print_debug("no more pacmen in sight")
            self.agent.reset_state()
            return True

    def choose_action(self):
        pacmen_pos = self.agent.pacmen_positions()
        #FIXME in case of multiple pacmen select the closest
        self.path = self.agent.closest_target(pacmen_pos)
        self.print_debug("Tracking! "+str(self.path.end))
        return self.path.pop()[0]

class BlockState(NinjaState):
    def __init__(self, agent):
        NinjaState.__init__(self, agent)

    def pre_state_change(self):
        if self.agent.pacman_is_close():
            self.print_debug("pacmen in sight, engage pursuit!")
            self.agent.state = PursuitState(self.agent)
            return True
        if self.agent.food_was_eaten:
            self.print_debug("food was eaten elsewhere, blocking wrong food")
            self.agent.state = HunterState(self.agent)
            return True

    def choose_action(self):
        self.select_block_path()
        if len(self.path) is not 0:
            return self.path.pop()[0]
        else:
            return Directions.STOP

    def select_block_path(self):
        self.print_debug("selecting new block path")
        self.path = self.agent.closest_target_for_enemy()

class Ninja(BasicAgent):

    def __init__(self, index, village, debug=False, rank=Genin):
        BasicAgent.__init__(self, index)
        self.debug = debug
        self.village = village
        self.village.recruit(self)
        self.match_start_rank = rank
        self.max_length = 23
        self.min_length = 2

    def set_Jonin(self):
        self.paranoia_distance=10
        self.run_distance=2
        self.preferd_state=HungryState
        self.halt=2
        self.rank = Jonin
        self.print_debug("is now"+self.rank)

    def set_Genin(self):
        self.paranoia_distance=7
        self.run_distance=4
        self.preferd_state=HungryState
        self.halt=4
        self.rank = Genin
        self.print_debug("is now"+self.rank)

    def set_Anbu(self):
        self.paranoia_distance=100
        self.run_distance=2
        self.preferd_state=HunterState
        self.halt=1
        self.rank = Anbu
        self.print_debug("is now"+self.rank)

    def set_Choji(self):
        self.paranoia_distance=7
        self.run_distance=0
        self.preferd_state=BlockState
        self.halt=0
        self.rank = Choji
        self.print_debug("is now"+self.rank)


    def register_initial_state(self, initial_game_state):
        self.__getattribute__('set_'+self.match_start_rank)()

        # store agent positions and actios, and game state.
        BasicAgent.register_initial_state(self, initial_game_state)
        self.game_state_stack = [initial_game_state]
        self.position_stack = [self.game_state.getInitialAgentPosition(self.index)]
        self.action_stack = [Directions.STOP]
        self.graph = uv.Graph(self.game_state.getWalls())
        self.protect_stack = [set(self.protect_list)]

        self.opponent_indices = self.game_state.getBlueTeamIndices() \
            if self.is_red \
            else self.game_state.getRedTeamIndices()

        # find the dividing line
        # for red this is the last line
        # for blue this is the first
        half_x = self.graph.width/2
        if self.is_red:
            half_x -= 1

        # find the nodes that belong to us and the enemy
        self.our_nodes = []
        self.opp_nodes = []
        for node in self.graph.nodes.keys():
            if (self.is_red and node[0] <= half_x) or \
                (self.is_blue and node[0] >= half_x):
                self.our_nodes.append(node)
            else:
                self.opp_nodes.append(node)

        # find the border nodes:
        self.border_nodes = []
        for y in range(self.graph.height):
            if (half_x, y) in self.graph.nodes.keys():
                self.border_nodes.append((half_x, y))

        self.border_nodes.sort()

        #FIXME this might throw index errors
        if self.index < 2:
            self.flee_point = self.border_nodes[len(self.border_nodes)/3-1]
        else:
            self.flee_point = self.border_nodes[len(self.border_nodes)/3*2+1]

        self.print_debug("flee_point: "+str(self.flee_point))

        if self.preferd_state is HungryState:
            # closest food to the flee_point
            path = self.graph.bfs_closest(self.flee_point, self.food_list)
            # path from position to that food
            food_path = self.closest_target([path.end])
            # initialize the HungryState
            self.state = HungryState(self, path = food_path)
        elif self.preferd_state is HunterState:
            self.state = HunterState(self, path=self.closest_target_for_enemy())
        else:
            self.state = self.preferd_state(self)

    @property
    def previous_action(self):
        return self.action_stack[-1]

    @property
    def game_state(self):
        return self.game_state_stack[-1]

    @property
    def legal_actions(self):
        return self.game_state.getLegalActions(self.index)

    @property
    def position(self):
        return self.game_state.getAgentPosition(self.index)

    @property
    def food_list(self):
        return self.remove_bogus(self.get_food(self.game_state).asList())

    @property
    def protect_list(self):
        return self.remove_bogus(self.get_food(self.game_state,
            enemy_food=False).asList())

    def remove_bogus(self, tokens):
        return [(x, y) for x, y in tokens if self.game_state.hasFood(x, y)]

    @property
    def is_killed(self):
        """Return True if the agent has been eaten on the previous turn."""
        if len(self.position_stack) > 2:
            return self.position_stack[-1] == self.position_stack[0]

    @property
    def is_pacman(self):
        return self.game_state.getAgentState(self.index).isPacman

    @property
    def is_ghost(self):
        return not self.is_pacman

    @property
    def is_blue(self):
        return not self.is_red

    @property
    def random_action(self):
        """ really choose a totally random action except STOP"""
        return random.choice(self.legal_actions)

    def print_debug(self, message):
        if self.debug:
            print "Ninja:"+str(self.index), "reports:", message

    def random_target(self, targets):
        """ from a list of possible targets select one at random """
        return self.graph.bfs(self.position, random.choice(targets))

    def closest_target(self, targets):
        """ from a list of possible targets select the closest"""
        return self.graph.bfs_closest(self.position, targets)

    def closest_target_for_enemy(self):
        enemy_start = \
        self.game_state.getInitialAgentPosition(self.opponent_indices[0])
        closest_target_for_enemy = self.graph.bfs_closest(enemy_start,
                self.protect_list).positions[-2]
        return self.graph.bfs(self.position, closest_target_for_enemy)


    def nodes_close_to_ghosts(self):
        """ get all nodes that are within running distance of ghosts """
        return self.graph.get_nodes_within_distance(self.ghost_positions(),
                self.run_distance)

    def reset_state(self):
        """ reset to prefered state """
        self.print_debug("resetting to prefered state")
        self.state = self.preferd_state(self)

    def pacman_is_close(self):
        return self.pacman_within_range(self.paranoia_distance)

    def ghost_is_close(self):
        return self.ghost_within_range(self.paranoia_distance)


    def enemy_is_close(self):
        return self.enemy_within_range(self.distances_to_enemy(self.enemy_positions()), self.paranoia_distance)


    def pacman_is_real_close(self):
        return self.pacman_within_range(self.run_distance)

    def ghost_is_real_close(self):
        return self.ghost_within_range(self.run_distance)

    def enemy_is_real_close(self):
        return self.enemy_within_range(self.distances_to_enemy(self.enemy_positions()), self.run_distance)


    def pacmen_positions(self):
        return self.enemy_positions(not_within_nodes=self.opp_nodes)

    def ghost_positions(self):
        return self.enemy_positions(not_within_nodes=self.our_nodes)

    def enemy_positions(self, not_within_nodes=[]):
        return [self.game_state.getAgentPosition(i) for i in self.opponent_indices
                if
                self.game_state.getAgentPosition(i) is not None
                and
                self.game_state.getAgentPosition(i) not in not_within_nodes]


    def enemy_info(self):
        return [(i, self.game_state.getAgentPosition(i)) for i in
                self.opponent_indices if
                self.game_state.getAgentPosition(i) is not None]

    def pacman_within_range(self, target_range):
        return self.enemy_within_range(self.distances_to_pacmen(), target_range)

    def ghost_within_range(self, target_range):
        return self.enemy_within_range(self.distances_to_ghosts(), target_range)

    def enemy_within_range(self, distances, target_range ):
        return any([d < target_range for d in distances])


    def distances_to_pacmen(self):
        return self.distances_to_enemy(self.pacmen_positions())

    def distances_to_ghosts(self):
        return self.distances_to_enemy(self.ghost_positions())

    def distances_to_enemy(self, enemy_positions , metric="maze"):
        if metric == "manhatten":
            return [uv.manhatten_distance(self.position, enemy_pos) for
                    enemy_pos in enemy_positions]
        elif metric == "maze":
            return [len(self.graph.bfs(self.position, enemy_pos)) for enemy_pos
                    in enemy_positions]

    def ghost_in_path(self, path):
        return any([pos in path.positions for pos in self.ghost_positions()])

    @property
    def food_was_eaten(self):
        if len(self.protect_stack) > 2:
            return len(self.protect_stack[-2]) is not len(self.protect_stack[-1])

    @property
    def eaten_food(self):
        return list(self.protect_stack[-2].difference(self.protect_stack[-1]))

    def detect_cycle(self, length):
        """ detect cycles in a position list """
        if len(self.position_stack) > 2*length:
            return self.position_stack[-length:] == self.position_stack[-2*length:-length]

    def detect_cycles(self):
        for length in range(self.min_length, self.max_length+1):
            if self.detect_cycle(length):
                return True


    def choose_action(self, game_state):
        """ store some information about the game, and choose the next action
        """
        self.game_state_stack.append(game_state)
        self.position_stack.append(self.position)
        self.protect_stack.append(set(self.protect_list))

        if self.is_killed:
            self.reset_state()

        while self.state.pre_state_change():
            # loop with side effect bad style
            pass

        # get next action
        action = self.state.choose_action()
        if action is None:
            # If the state really can't com with anything
            # If debug is disabled don't crash, instead fall through
            # to next case and return a random action
            if self.debug:
                raise RuntimeError("choose_action returned None")
        if action not in self.legal_actions:
            # this is a hack in case something goes wrong
            self.say("INAVALID ACTION")
            self.print_debug("INVALID ACTION")
            action = self.random_action
            self.reset_state

        self.state.post_state_change()

        self.action_stack.append(action)
        return action


