import unittest
from testingTools import get_mock_game
from basic_agents import BasicAgent
from game import Directions

class RemoteControlledAgent(BasicAgent):
    def __init__(self, index):
        super(RemoteControlledAgent, self).__init__(index)
        self.next_move = Directions.STOP
        
    def set_next_move(self, move):
        self.next_move = move
        
    def choose_action(self, game_state):
        return self.next_move

def print_pos(game, a1, a2):
    print 'next_turn', game._agentIndex
    print a1.index, game.state.getAgentPosition(a1.index)
    print a2.index, game.state.getAgentPosition(a2.index)
    
class PacmanNotEatingFoodBug(unittest.TestCase):
    """Bug: if a ghost crosses the border to land on a food point,
    it does not get eaten!
    """
    def test_pacman_eat_food(self):
        layout = """%%%%%%
                    % 1.2%
                    %%%%%%
                    """
        pacman = RemoteControlledAgent(0)
        _, game = get_mock_game(layout, [pacman, BasicAgent(1)])
        game._init_run()
        
        pacman.set_next_move(Directions.EAST)

        # the food should be there
        self.assertTrue(game.state.hasFood(3, 1))
        game._one_game_step()
        # the food should be gone
        self.assertFalse(game.state.hasFood(3, 1))

class FoodGhostPacmanTestCase(unittest.TestCase):
    """Test that the following situation is reliably handled by the game:
    a ghost is on top of a food dot; a pacman goes on the same square;
    the pacman should die *before* eating the food."""
    
    def test_foodghostpacman(self):
        layout = """%%%%%%%%%%%%%%
                    %1       .2 .%
                    %%%%%%%%%%%%%%
                    """
        pacman = RemoteControlledAgent(0)
        ghost = RemoteControlledAgent(1)
        _, game = get_mock_game(layout, [pacman, ghost])
        game._init_run()
        
        # move the pacman and ghost in the right position
        pacman.set_next_move(Directions.EAST)
        ghost.set_next_move(Directions.WEST)
        game._one_game_step()
        game._one_game_step()
        print_pos(game, pacman, ghost)

        self.assertEqual(game.state.getAgentPosition(1), (9,1))
        
        # do the crucial move: the pacman should die
        pacman.set_next_move(Directions.EAST)
        ghost.set_next_move(Directions.STOP)
        
        # 6 moves for the red to get close to the food
        for _ in range(6):
            game._one_game_step() # red moves
            game._one_game_step() # blue moves
            print_pos(game, pacman, ghost)
        # the red should be close to the food
        self.assertEqual(game.state.getAgentPosition(0), (8,1))
        # the food should still be there
        self.assertTrue(game.state.hasFood(9, 1))
        
        # pacman moves
        game._one_game_step()
        print 'crucial move'
        print_pos(game, pacman, ghost)
       
        # the ghost should be on the food
        self.assertEqual(game.state.getAgentPosition(1), (9,1))
        # the food should still be there
        # FIXME: we'll leave this behavior during the course, we'll have
        # to change it afterwards
        #self.assertTrue(game.state.hasFood(9, 1))
        # the pacman should be back at the start case
        self.assertEqual(game.state.getAgentPosition(0), (1,1))

if __name__ == '__main__':
    unittest.main()